@@ -41,6 +41,9 @@
 
 #include "ikea.h"
 
+#define EXCH_AGGR_MODE		0
+#define EXCH_MAIN_MODE		1
+
 #define AUTH_HYBRID_RSA_XAUTH	0
 #define AUTH_MUTUAL_RSA_XAUTH	1
 #define AUTH_MUTUAL_PSK_XAUTH	2
@@ -176,13 +179,13 @@ void site::init()
 {
 	comboBoxConfigMethod->setCurrentItem( 1 );
 
-	lineEditAddress->setInputMask( "00D . 00D . 00D . 00D" );
+	lineEditAddress->setInputMask( "009 . 009 . 009 . 009" );
 	lineEditAddress->setText( "0.0.0.0" );
 
-	lineEditNetmask->setInputMask( "00D . 00D . 00D . 00D" );
+	lineEditNetmask->setInputMask( "009 . 009 . 009 . 009" );
 	lineEditNetmask->setText( "255.255.255.255" );
 
-	lineEditDNSServer->setInputMask( "00D . 00D . 00D . 00D" );
+	lineEditDNSServer->setInputMask( "009 . 009 . 009 . 009" );
 	lineEditDNSServer->setText( "0.0.0.0" );
 
 #ifdef OPT_NATT
@@ -203,11 +206,11 @@ void site::init()
 
 	// update dialog
 
-	Update();
-	UpdateAuth();
-	UpdateExchange();
-	UpdateCipher();
-	UpdateTransform();
+	UpdateGeneral();
+	UpdateAuthentication();
+	UpdatePhase1();
+	UpdatePhase2();
+	UpdatePolicy();
 }
 
 void site::ModPolicy()
@@ -470,11 +473,11 @@ bool site::Load( CONFIG & config )
 	if( config.get_string( "phase1-exchange",
 		text, MAX_CONFSTRING, 0 ) )
 	{
-		if( !strcmp( text, "main" ) )
-			comboBoxP1Exchange->setCurrentItem( 0 );
-
 		if( !strcmp( text, "aggressive" ) )
-			comboBoxP1Exchange->setCurrentItem( 1 );
+			comboBoxP1Exchange->setCurrentItem( EXCH_AGGR_MODE );
+
+		if( !strcmp( text, "main" ) )
+			comboBoxP1Exchange->setCurrentItem( EXCH_MAIN_MODE );
 	}
 
 	// authentication mode ( default hybrid rsa xauth )
@@ -498,8 +501,6 @@ bool site::Load( CONFIG & config )
 			comboBoxAuthMethod->setCurrentItem( 4 );
 	}
 
-	UpdateAuth();
-
 	// local identity type
 	//
 	// NOTE : Requires phase1 exchange type & authentication mode
@@ -624,6 +625,15 @@ bool site::Load( CONFIG & config )
 	if( config.get_number( "phase1-life-kbytes", &numb ) )
 		lineEditP1LifeData->setText( QString::number( numb, 10 ) );
 
+	// phase1 checkpoint vendor option
+
+	numb = 1;
+	config.get_number( "vendor-chkpt-enable", &numb );
+	if( numb )
+		checkBoxCheckpointID->setChecked( true );
+	else
+		checkBoxCheckpointID->setChecked( false );
+
 	// phase2 trasform algorithm ( default auto )
 
 	if( config.get_string( "phase2-transform",
@@ -714,10 +724,11 @@ bool site::Load( CONFIG & config )
 
 	// update dialog
 
-	Update();
-	UpdateExchange();
-	UpdateCipher();
-	UpdateTransform();
+	UpdateGeneral();
+	UpdateAuthentication();
+	UpdatePhase1();
+	UpdatePhase2();
+	UpdatePolicy();
 
 	return true;
 }
@@ -1060,22 +1071,13 @@ bool site::Save( CONFIG & config )
 
 	// phase1 exchange type
 
-	if( !comboBoxP1Exchange->currentItem() )
-	{
-		// main mode
-
+	if( comboBoxP1Exchange->currentItem() == EXCH_AGGR_MODE )
 		config.set_string( "phase1-exchange",
-			"main",
-			strlen( "main" ) );
-	}
-	else
-	{
-		// aggressive mode
+			"aggressive", strlen( "aggressive" ) );
 
+	if( comboBoxP1Exchange->currentItem() == EXCH_MAIN_MODE )
 		config.set_string( "phase1-exchange",
-			"aggressive",
-			strlen( "aggressive" ) );
-	}
+			"main",	strlen( "main" ) );
 
 	// phase1 dh group
 
@@ -1112,6 +1114,13 @@ bool site::Save( CONFIG & config )
 	config.set_number( "phase1-life-kbytes",
 		lineEditP1LifeData->text().toLong() );
 
+	// phase1 Checkpoint vendor option
+
+	if( !checkBoxCheckpointID->isChecked() )
+		config.set_number( "vendor-chkpt-enable", 0 );
+	else
+		config.set_number( "vendor-chkpt-enable", 1 );
+
 	// phase2 trasform algorithm
 
 	config.set_string( "phase2-transform",
@@ -1276,9 +1285,10 @@ bool site::Verify()
 	return true;
 }
 
-
-void site::Update()
+void site::UpdateGeneral()
 {
+	QString addrmeth = comboBoxAddressMethod->currentText();
+
 	// auto configuration
 
 	long aconf = comboBoxConfigMethod->currentItem();
@@ -1301,7 +1311,13 @@ void site::Update()
 		comboBoxAddressMethod->insertItem( "Use existing adapter and current address" );
 	}
 
-	// local adapter mode
+	// adapter mode
+
+	combobox_setbytext( ( char * ) addrmeth.ascii(), comboBoxAddressMethod );
+
+	bool addrmeth_changed = false;
+	if( addrmeth != comboBoxAddressMethod->currentText() )
+		addrmeth_changed = true;
 
 	if( !comboBoxAddressMethod->currentItem() )
 	{
@@ -1317,7 +1333,8 @@ void site::Update()
 			case 1: // ike config push
 			case 2: // ike config pull
 				checkBoxAddressAuto->setEnabled( true );
-				checkBoxAddressAuto->setChecked( true );
+				if( addrmeth_changed )
+					checkBoxAddressAuto->setChecked( true );
 				break;
 
 			case 3:	// dhcp over ipsec
@@ -1352,6 +1369,14 @@ void site::Update()
 		lineEditNetmask->setEnabled( false );
 	}
 
+	UpdateClient();
+	UpdateNameResolution();
+}
+
+void site::UpdateClient()
+{
+	long aconf = comboBoxConfigMethod->currentItem();
+
 	// nat traversal mode
 
 	if( !comboBoxNATTMode->currentItem() )
@@ -1394,7 +1419,7 @@ void site::Update()
 		lineEditFragSize->setEnabled( true );
 	}
 
-	// client long banner
+	// login banner
 
 	if( aconf == 0 )
 	{
@@ -1403,6 +1428,11 @@ void site::Update()
 	}
 	else
 		checkBoxBanner->setEnabled( true );
+}
+
+void site::UpdateNameResolution()
+{
+	long aconf = comboBoxConfigMethod->currentItem();
 
 	// dns enabled
 
@@ -1444,174 +1474,9 @@ void site::Update()
 		textLabelDNSSuffix->setEnabled( false );
 		lineEditDNSSuffix->setEnabled( false );
 	}
-
-	// policy configuration
-
-	if( checkBoxPolicyAuto->isChecked() )
-	{
-		// automatic
-
-		listViewPolicies->setEnabled( false );
-
-		pushButtonPolicyAdd->setEnabled( false );
-		pushButtonPolicyMod->setEnabled( false );
-		pushButtonPolicyDel->setEnabled( false );
-	}
-	else
-	{
-		// manual
-
-		listViewPolicies->setEnabled( true );
-
-		pushButtonPolicyAdd->setEnabled( true );
-
-		// policy item selection
-
-		if( listViewPolicies->selectedItem() != NULL )
-		{
-			// have selection
-
-			pushButtonPolicyMod->setEnabled( true );
-			pushButtonPolicyDel->setEnabled( true );
-		}
-		else
-		{
-			// no selection
-
-			pushButtonPolicyMod->setEnabled( false );
-			pushButtonPolicyDel->setEnabled( false );
-		}
-	}
 }
 
-void site::UpdateExchange()
-{
-	// exchange mode
-
-	if( !comboBoxP1Exchange->currentItem() )
-	{
-		// main mode ( auto allowed )
-
-		QString text = comboBoxP1DHGroup->currentText();
-
-		comboBoxP1DHGroup->clear();
-		comboBoxP1DHGroup->insertItem( "auto" );
-		comboBoxP1DHGroup->insertItem( "group 1" );
-		comboBoxP1DHGroup->insertItem( "group 2" );
-		comboBoxP1DHGroup->insertItem( "group 5" );
-		comboBoxP1DHGroup->insertItem( "group 14" );
-		comboBoxP1DHGroup->insertItem( "group 15" );
-
-		combobox_setbytext( ( char * ) text.ascii(), comboBoxP1DHGroup );
-	}
-	else
-	{
-		// aggressive mode ( auto not allowed )
-
-		QString text = comboBoxP1DHGroup->currentText();
-
-		comboBoxP1DHGroup->clear();
-		comboBoxP1DHGroup->insertItem( "group 1" );
-		comboBoxP1DHGroup->insertItem( "group 2" );
-		comboBoxP1DHGroup->insertItem( "group 5" );
-		comboBoxP1DHGroup->insertItem( "group 14" );
-		comboBoxP1DHGroup->insertItem( "group 15" );
-
-		combobox_setbytext( ( char * ) text.ascii(), comboBoxP1DHGroup );
-	}
-
-	UpdateAuth();
-}
-
-void site::UpdateCipher()
-{
-	// chipher type
-
-	QString text = comboBoxP1Keylen->currentText();
-
-	switch( comboBoxP1Cipher->currentItem() )
-	{
-		case 1: // aes
-		{
-			comboBoxP1Keylen->setEnabled( true );
-			comboBoxP1Keylen->clear();
-			comboBoxP1Keylen->insertItem( "auto" );
-			comboBoxP1Keylen->insertItem( "128" );
-			comboBoxP1Keylen->insertItem( "192" );
-			comboBoxP1Keylen->insertItem( "256" );
-			break;
-		}
-
-		case 2: // blowfish
-		{
-			comboBoxP1Keylen->setEnabled( true );
-			comboBoxP1Keylen->clear();
-			comboBoxP1Keylen->insertItem( "auto" );
-			for( long b = 128; b <= 256; b += 8 )
-				comboBoxP1Keylen->insertItem(
-					QString::number( b, 10 ) );
-			break;
-		}
-
-		default: // all others
-		{
-			comboBoxP1Keylen->setCurrentItem( 0 );
-			comboBoxP1Keylen->setEnabled( false );
-			break;
-		}
-	}
-
-	if( !comboBoxP1Cipher->currentItem() )
-		comboBoxP1Keylen->setCurrentItem( 0 );
-	else
-		combobox_setbytext( ( char * ) text.ascii(), comboBoxP1Keylen );
-}
-         
-void site::UpdateTransform()
-{
-	// chipher type
-
-	QString text = comboBoxP2Keylen->currentText();
-
-	switch( comboBoxP2Transform->currentItem() )
-	{
-		case 1: // aes
-		{
-			comboBoxP2Keylen->setEnabled( true );
-			comboBoxP2Keylen->clear();
-			comboBoxP2Keylen->insertItem( "auto" );
-			comboBoxP2Keylen->insertItem( "128" );
-			comboBoxP2Keylen->insertItem( "192" );
-			comboBoxP2Keylen->insertItem( "256" );
-			break;
-		}
-
-		case 2: // blowfish
-		{
-			comboBoxP2Keylen->setEnabled( true );
-			comboBoxP2Keylen->clear();
-			comboBoxP2Keylen->insertItem( "auto" );
-			for( long b = 128; b <= 256; b += 8 )
-				comboBoxP2Keylen->insertItem(
-					QString::number( b, 10 ) );
-			break;
-		}
-
-		default: // all others
-		{
-			comboBoxP2Keylen->setCurrentItem( 0 );
-			comboBoxP2Keylen->setEnabled( false );
-			break;
-		}
-	}
-
-	if( !comboBoxP2Transform->currentItem() )
-		comboBoxP2Keylen->setCurrentItem( 0 );
-	else
-		combobox_setbytext( ( char * ) text.ascii(), comboBoxP2Keylen );
-}
-
-void site::UpdateAuth()
+void site::UpdateAuthentication()
 {
 	// authentication method
 
@@ -1643,14 +1508,7 @@ void site::UpdateAuth()
 		{
 			comboBoxLocalIDType->clear();
 
-			if( !comboBoxP1Exchange->currentItem() )
-			{
-				// main mode
-
-				comboBoxLocalIDType->insertItem( IDTXT_ASN1 );
-				comboBoxLocalIDType->insertItem( IDTXT_ADDR );
-			}
-			else
+			if( comboBoxP1Exchange->currentItem() == EXCH_AGGR_MODE )
 			{
 				// aggressive mode
 
@@ -1661,6 +1519,14 @@ void site::UpdateAuth()
 				comboBoxLocalIDType->insertItem( IDTXT_KEYID );
 			}
 
+			if( comboBoxP1Exchange->currentItem() == EXCH_MAIN_MODE )
+			{
+				// main mode
+
+				comboBoxLocalIDType->insertItem( IDTXT_ASN1 );
+				comboBoxLocalIDType->insertItem( IDTXT_ADDR );
+			}
+
 			break;
 		}
 
@@ -1669,13 +1535,7 @@ void site::UpdateAuth()
 		{
 			comboBoxLocalIDType->clear();
 
-			if( !comboBoxP1Exchange->currentItem() )
-			{
-				// main mode
-
-				comboBoxLocalIDType->insertItem( IDTXT_ADDR );
-			}
-			else
+			if( comboBoxP1Exchange->currentItem() == EXCH_AGGR_MODE )
 			{
 				// aggressive mode
 
@@ -1685,6 +1545,13 @@ void site::UpdateAuth()
 				comboBoxLocalIDType->insertItem( IDTXT_KEYID );
 			}
 
+			if( comboBoxP1Exchange->currentItem() == EXCH_MAIN_MODE )
+			{
+				// main mode
+
+				comboBoxLocalIDType->insertItem( IDTXT_ADDR );
+			}
+
 			break;
 		}
 	}
@@ -1709,14 +1576,7 @@ void site::UpdateAuth()
 		{
 			comboBoxRemoteIDType->clear();
 
-			if( !comboBoxP1Exchange->currentItem() )
-			{
-				// main mode
-
-				comboBoxRemoteIDType->insertItem( IDTXT_ASN1 );
-				comboBoxRemoteIDType->insertItem( IDTXT_ADDR );
-			}
-			else
+			if( comboBoxP1Exchange->currentItem() == EXCH_AGGR_MODE )
 			{
 				// aggressive mode
 
@@ -1727,6 +1587,14 @@ void site::UpdateAuth()
 				comboBoxRemoteIDType->insertItem( IDTXT_KEYID );
 			}
 
+			if( comboBoxP1Exchange->currentItem() == EXCH_MAIN_MODE )
+			{
+				// main mode
+
+				comboBoxRemoteIDType->insertItem( IDTXT_ASN1 );
+				comboBoxRemoteIDType->insertItem( IDTXT_ADDR );
+			}
+
 			break;
 		}
 
@@ -1735,13 +1603,7 @@ void site::UpdateAuth()
 		{
 			comboBoxRemoteIDType->clear();
 
-			if( !comboBoxP1Exchange->currentItem() )
-			{
-				// main mode
-
-				comboBoxRemoteIDType->insertItem( IDTXT_ADDR );
-			}
-			else
+			if( comboBoxP1Exchange->currentItem() == EXCH_AGGR_MODE )
 			{
 				// aggressive mode
 
@@ -1751,6 +1613,13 @@ void site::UpdateAuth()
 				comboBoxRemoteIDType->insertItem( IDTXT_KEYID );
 			}
 
+			if( comboBoxP1Exchange->currentItem() == EXCH_MAIN_MODE )
+			{
+				// main mode
+
+				comboBoxRemoteIDType->insertItem( IDTXT_ADDR );
+			}
+
 			break;
 		}
 	}
@@ -1930,6 +1799,171 @@ void site::SelectRemoteID()
 	UpdateRemoteID();
 }
 
+void site::UpdatePhase1()
+{
+	// exchange mode
+
+	if( comboBoxP1Exchange->currentItem() == EXCH_AGGR_MODE )
+	{
+		// aggressive mode ( auto not allowed )
+
+		QString text = comboBoxP1DHGroup->currentText();
+
+		comboBoxP1DHGroup->clear();
+		comboBoxP1DHGroup->insertItem( "group 1" );
+		comboBoxP1DHGroup->insertItem( "group 2" );
+		comboBoxP1DHGroup->insertItem( "group 5" );
+		comboBoxP1DHGroup->insertItem( "group 14" );
+		comboBoxP1DHGroup->insertItem( "group 15" );
+
+		combobox_setbytext( ( char * ) text.ascii(), comboBoxP1DHGroup );
+	}
+
+	if( comboBoxP1Exchange->currentItem() == EXCH_MAIN_MODE )
+	{
+		// main mode ( auto allowed )
+
+		QString text = comboBoxP1DHGroup->currentText();
+
+		comboBoxP1DHGroup->clear();
+		comboBoxP1DHGroup->insertItem( "auto" );
+		comboBoxP1DHGroup->insertItem( "group 1" );
+		comboBoxP1DHGroup->insertItem( "group 2" );
+		comboBoxP1DHGroup->insertItem( "group 5" );
+		comboBoxP1DHGroup->insertItem( "group 14" );
+		comboBoxP1DHGroup->insertItem( "group 15" );
+
+		combobox_setbytext( ( char * ) text.ascii(), comboBoxP1DHGroup );
+	}
+
+	// chipher type
+
+	QString text = comboBoxP1Keylen->currentText();
+
+	switch( comboBoxP1Cipher->currentItem() )
+	{
+		case 1: // aes
+		{
+			comboBoxP1Keylen->setEnabled( true );
+			comboBoxP1Keylen->clear();
+			comboBoxP1Keylen->insertItem( "auto" );
+			comboBoxP1Keylen->insertItem( "128" );
+			comboBoxP1Keylen->insertItem( "192" );
+			comboBoxP1Keylen->insertItem( "256" );
+			break;
+		}
+
+		case 2: // blowfish
+		{
+			comboBoxP1Keylen->setEnabled( true );
+			comboBoxP1Keylen->clear();
+			comboBoxP1Keylen->insertItem( "auto" );
+			for( long b = 128; b <= 256; b += 8 )
+				comboBoxP1Keylen->insertItem(
+					QString::number( b, 10 ) );
+			break;
+		}
+
+		default: // all others
+		{
+			comboBoxP1Keylen->setCurrentItem( 0 );
+			comboBoxP1Keylen->setEnabled( false );
+			break;
+		}
+	}
+
+	if( !comboBoxP1Cipher->currentItem() )
+		comboBoxP1Keylen->setCurrentItem( 0 );
+	else
+		combobox_setbytext( ( char * ) text.ascii(), comboBoxP1Keylen );
+
+	UpdateAuthentication();
+}
+
+void site::UpdatePhase2()
+{
+	// transform type
+
+	QString text = comboBoxP2Keylen->currentText();
+
+	switch( comboBoxP2Transform->currentItem() )
+	{
+		case 1: // aes
+		{
+			comboBoxP2Keylen->setEnabled( true );
+			comboBoxP2Keylen->clear();
+			comboBoxP2Keylen->insertItem( "auto" );
+			comboBoxP2Keylen->insertItem( "128" );
+			comboBoxP2Keylen->insertItem( "192" );
+			comboBoxP2Keylen->insertItem( "256" );
+			break;
+		}
+
+		case 2: // blowfish
+		{
+			comboBoxP2Keylen->setEnabled( true );
+			comboBoxP2Keylen->clear();
+			comboBoxP2Keylen->insertItem( "auto" );
+			for( long b = 128; b <= 256; b += 8 )
+				comboBoxP2Keylen->insertItem(
+					QString::number( b, 10 ) );
+			break;
+		}
+
+		default: // all others
+		{
+			comboBoxP2Keylen->setCurrentItem( 0 );
+			comboBoxP2Keylen->setEnabled( false );
+			break;
+		}
+	}
+
+	if( !comboBoxP2Transform->currentItem() )
+		comboBoxP2Keylen->setCurrentItem( 0 );
+	else
+		combobox_setbytext( ( char * ) text.ascii(), comboBoxP2Keylen );
+}
+
+void site::UpdatePolicy()
+{
+	// policy configuration
+
+	if( checkBoxPolicyAuto->isChecked() )
+	{
+		// automatic
+
+		listViewPolicies->setEnabled( false );
+
+		pushButtonPolicyAdd->setEnabled( false );
+		pushButtonPolicyMod->setEnabled( false );
+		pushButtonPolicyDel->setEnabled( false );
+	}
+	else
+	{
+		// manual
+
+		listViewPolicies->setEnabled( true );
+
+		pushButtonPolicyAdd->setEnabled( true );
+
+		// policy item selection
+
+		if( listViewPolicies->selectedItem() != NULL )
+		{
+			// have selection
+
+			pushButtonPolicyMod->setEnabled( true );
+			pushButtonPolicyDel->setEnabled( true );
+		}
+		else
+		{
+			// no selection
+
+			pushButtonPolicyMod->setEnabled( false );
+			pushButtonPolicyDel->setEnabled( false );
+		}
+	}
+}
 
 void site::InputCAFile()
 {
@@ -1986,3 +2020,4 @@ void site::VerifyAccept()
 
 	return;
 }
+
