@@ -101,7 +101,7 @@ bool _IKEC::log( long code, const char * format, ... )
 	memset( buff, 0, sizeof( buff ) );
 	va_list list;
 	va_start( list, format );
-	vsprintf( buff, format, list );
+	vsnprintf( buff, sizeof( buff ), format, list );
 
 	QApplication::postEvent( r, new StatusEvent( buff, code ) );
 
@@ -141,7 +141,7 @@ void _IKEC::run()
 	while( host && * host == ' ' )
 		host++;
 
-	if( isdigit( host[ 0 ] ) )
+	if( inet_addr( host ) != INADDR_NONE )
 	{
 		peer.saddr.saddr4.sin_family = AF_INET;
 		peer.saddr.saddr4.sin_addr.s_addr = inet_addr( host );
@@ -762,6 +762,13 @@ void _IKEC::run()
 		peer.plcy_mode = POLICY_MODE_COMPAT;
 	}
 
+	// vendor compatibility options
+
+	numb = 0;
+	config.get_number( "vendor-chkpt-enable", &numb );
+	if( numb )
+		xconf.opts |= IPSEC_OPTS_VEND_CHKPT;
+
 	//
 	// ---------- CONNECT TO IKED ----------
 	//
@@ -786,7 +793,7 @@ void _IKEC::run()
 
 	active = true;
 
-	QApplication::postEvent( r, new RunningEvent( true, host ) );
+	QApplication::postEvent( r, new RunningEvent( true ) );
 
 	//
 	// send the peer configuration message
@@ -900,28 +907,32 @@ void _IKEC::run()
 
 	// client id data
 
-	config.get_string( "ident-client-data", btext, 0 );
-	msg.set_cfgstr( CFGSTR_CRED_LID, &btext );
-	result = ikei.send_message( msg, &msgres );
-
-	if( ( result != IPCERR_OK ) || ( msgres != IKEI_RESULT_OK ) )
+	if( config.get_string( "ident-client-data", btext, 0 ) )
 	{
-		log( STATUS_FAIL, "local id config failed\n" );
-		goto config_failed;
+		msg.set_cfgstr( CFGSTR_CRED_LID, &btext );
+		result = ikei.send_message( msg, &msgres );
+
+		if( ( result != IPCERR_OK ) || ( msgres != IKEI_RESULT_OK ) )
+		{
+			log( STATUS_FAIL, "local id config failed\n" );
+			goto config_failed;
+		}
 	}
 
 	log( STATUS_INFO, "local id configured\n" );
 
 	// server id data
 
-	config.get_string( "ident-server-data", btext, 0 );
-	msg.set_cfgstr( CFGSTR_CRED_RID, &btext );
-	result = ikei.send_message( msg, &msgres );
-
-	if( ( result != IPCERR_OK ) || ( msgres != IKEI_RESULT_OK ) )
+	if( config.get_string( "ident-server-data", btext, 0 ) )
 	{
-		log( STATUS_FAIL, "remote id config failed\n" );
-		goto config_failed;
+		msg.set_cfgstr( CFGSTR_CRED_RID, &btext );
+		result = ikei.send_message( msg, &msgres );
+
+		if( ( result != IPCERR_OK ) || ( msgres != IKEI_RESULT_OK ) )
+		{
+			log( STATUS_FAIL, "remote id config failed\n" );
+			goto config_failed;
+		}
 	}
 
 	log( STATUS_INFO, "remote id configured\n" );
@@ -1286,7 +1297,7 @@ void _IKEC::run()
 
 	active = false;
 
-	QApplication::postEvent( r, new RunningEvent( false, "" ) );
+	QApplication::postEvent( r, new RunningEvent( false ) );
 
 	return;
 }
