@@ -55,85 +55,24 @@ long _IKED::process_phase1_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 
 	//
 	// make sure we are not dealing
-	// whith a mature or dead sa
+	// with an sa marked for delete
 	//
 
 	if( ph1->status() == XCH_STATUS_DEAD )
 	{
-		log.txt( LLOG_ERROR, "!! : phase1 packet ignored ( sa marked for death )\n" );
+		log.txt( LLOG_ERROR, "!! : phase1 packet ignored ( phase1 marked for death )\n" );
 		return LIBIKE_OK;
 	}
 
+	//
+	// make sure we are not dealing
+	// with a mature sa
+	//
+
 	if( ph1->status() >= XCH_STATUS_MATURE )
 	{
-		log.txt( LLOG_ERROR, "!! : phase1 packet ignored ( sa already mature )\n" );
-		return LIBIKE_OK;
-	}
-
-	//
-	// check for duplicate exchange packets.
-	// if we don't discard these before our
-	// processing, we run the risk of errors
-	// assiciated with erroneous data caused
-	// by decrypting packets using invalid
-	// cipher initialization vector data.
-	//
-
-	bool duplicate = false;
-
-	if( ph1->exchange == ISAKMP_EXCH_IDENT_PROTECT )
-	{
-		switch( payload )
-		{
-			case ISAKMP_PAYLOAD_SA:
-				if( ph1->xstate & XSTATE_RECV_SA )
-					duplicate = true;
-				break;
-
-			case ISAKMP_PAYLOAD_KEX:
-				if( ph1->xstate & XSTATE_RECV_KE )
-					duplicate = true;
-				break;
-
-			case ISAKMP_PAYLOAD_IDENT:
-				if( ph1->xstate & XSTATE_RECV_ID )
-					duplicate = true;
-				break;
-
-			case ISAKMP_PAYLOAD_HASH:
-				if( ph1->xstate & XSTATE_RECV_HA )
-					duplicate = true;
-				break;
-		}
-	}
-
-	if( ph1->exchange == ISAKMP_EXCH_AGGRESSIVE )
-	{
-		switch( payload )
-		{
-			case ISAKMP_PAYLOAD_SA:
-				if( ph1->xstate & XSTATE_RECV_SA )
-					duplicate = true;
-				break;
-
-			case ISAKMP_PAYLOAD_CERT:
-				if( ph1->xstate & XSTATE_RECV_CT )
-					duplicate = true;
-				break;
-
-			case ISAKMP_PAYLOAD_HASH:
-				if( ph1->xstate & XSTATE_RECV_HA )
-					duplicate = true;
-				break;
-		}
-	}
-
-	if( duplicate )
-	{
-		log.txt( LLOG_ERROR,
-			"!! : phase1 packet ignored ( duplicate leading %s payload )\n",
-			find_name( NAME_PAYLOAD, payload ) );
-
+		log.txt( LLOG_ERROR, "!! : phase1 packet ignored, resending last packet ( phase1 already mature )\n" );
+		ph1->resend();
 		return LIBIKE_OK;
 	}
 
@@ -143,9 +82,8 @@ long _IKED::process_phase1_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 
 	if( packet_ike_decrypt( ph1, packet, &ph1->iv ) != LIBIKE_OK )
 	{
-		log.txt( LLOG_ERROR,
-			"!! : phase1 packet ignored ( packet decryption error )\n" );
-
+		log.txt( LLOG_ERROR, "!! : phase1 packet ignored, resending last packet ( packet decryption error )\n" );
+		ph1->resend();
 		return LIBIKE_OK;
 	}
 
@@ -356,19 +294,12 @@ long _IKED::process_phase1_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 			case ISAKMP_PAYLOAD_NAT_VXX_DISC:
 			case ISAKMP_PAYLOAD_NAT_RFC_DISC:
 			{
-				if( !( ph1->xstate & XSTATE_RECV_NDL ) )
-				{
-					result = payload_get_natd( packet, ph1->natd_rd, ph1->hash_size );
-					ph1->xstate |= XSTATE_RECV_NDL;
-					break;
-				}
+				BDATA natd;
+				result = payload_get_natd( packet, natd, ph1->hash_size );
+				if( result == LIBIKE_OK )
+					ph1->natd_hash_r.add( natd );
 
-				if( !( ph1->xstate & XSTATE_RECV_NDR ) )
-				{
-					result = payload_get_natd( packet, ph1->natd_rs, ph1->hash_size );
-					ph1->xstate |= XSTATE_RECV_NDR;
-					break;
-				}
+				ph1->xstate |= XSTATE_RECV_ND;
 
 				break;
 			}
@@ -424,7 +355,7 @@ long _IKED::process_phase1_recv( IDB_PH1 * ph1, PACKET_IKE & packet, unsigned ch
 		//
 
 		if( packet.get_payload_left() )
-			log.txt( LLOG_ERROR, "XX : warning, unprocessed payload data !!!\n" );
+			log.txt( LLOG_ERROR, "!! : unprocessed payload data\n" );
 
 		//
 		// now that we have decoded the payload,
@@ -627,8 +558,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 				if( ph1->natt_version != IPSEC_NATT_NONE )
 				{
 					phase1_gen_natd( ph1 );
-					payload_add_natd( packet, ph1->natd_ld, ph1->natt_pldtype );
-					payload_add_natd( packet, ph1->natd_ls, ISAKMP_PAYLOAD_NONE );
+					phase1_add_natd( ph1, packet, ISAKMP_PAYLOAD_NONE );
 				}
 
 				packet.done();
@@ -722,7 +652,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 				// encrypt and send packet
 				//
 
-				packet_ike_send( ph1, ph1, packet, true );
+				packet_ike_send( ph1, ph1, packet, false );
 
 				ph1->xstate |= XSTATE_SENT_ID;
 			}
@@ -815,8 +745,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 				if( ph1->natt_version != IPSEC_NATT_NONE )
 				{
 					phase1_gen_natd( ph1 );
-					payload_add_natd( packet, ph1->natd_ld, ph1->natt_pldtype );
-					payload_add_natd( packet, ph1->natd_ls, ISAKMP_PAYLOAD_NONE );
+					phase1_add_natd( ph1, packet, ISAKMP_PAYLOAD_NONE );
 				}
 
 				packet.done();
@@ -887,7 +816,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 						// send packet
 						//
 
-						packet_ike_send( ph1, ph1, packet, true );
+						packet_ike_send( ph1, ph1, packet, false );
 
 						ph1->xstate |= XSTATE_SENT_ID;
 						ph1->xstate |= XSTATE_SENT_HA;
@@ -945,7 +874,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 						// send packet
 						//
 
-						packet_ike_send( ph1, ph1, packet, true );
+						packet_ike_send( ph1, ph1, packet, false );
 
 						ph1->xstate |= XSTATE_SENT_ID;
 						ph1->xstate |= XSTATE_SENT_CT;
@@ -1092,8 +1021,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 							if( ph1->natt_version != IPSEC_NATT_NONE )
 							{
 								phase1_gen_natd( ph1 );
-								payload_add_natd( packet, ph1->natd_ld, ph1->natt_pldtype );
-								payload_add_natd( packet, ph1->natd_ls, ISAKMP_PAYLOAD_NONE );
+								phase1_add_natd( ph1, packet, ISAKMP_PAYLOAD_NONE );
 							}
 
 							packet.done();
@@ -1102,7 +1030,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 							// send packet
 							//
 
-							packet_ike_send( ph1, ph1, packet, true );
+							packet_ike_send( ph1, ph1, packet, false );
 
 							ph1->xstate |= XSTATE_SENT_HA;
 						}
@@ -1143,8 +1071,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 							if( ph1->natt_version != IPSEC_NATT_NONE )
 							{
 								phase1_gen_natd( ph1 );
-								payload_add_natd( packet, ph1->natd_ld, ph1->natt_pldtype );
-								payload_add_natd( packet, ph1->natd_ls, ISAKMP_PAYLOAD_NONE );
+								phase1_add_natd( ph1, packet, ISAKMP_PAYLOAD_NONE );
 							}
 
 							packet.done();
@@ -1153,7 +1080,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 							// send packet
 							//
 
-							packet_ike_send( ph1, ph1, packet, true );
+							packet_ike_send( ph1, ph1, packet, false );
 
 							ph1->xstate |= XSTATE_SENT_CT;
 							ph1->xstate |= XSTATE_SENT_SI;
@@ -1258,8 +1185,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 				if( ph1->natt_version != IPSEC_NATT_NONE )
 				{
 					phase1_gen_natd( ph1 );
-					payload_add_natd( packet, ph1->natd_ld, ph1->natt_pldtype );
-					payload_add_natd( packet, ph1->natd_ls, ISAKMP_PAYLOAD_NONE );
+					phase1_add_natd( ph1, packet, ISAKMP_PAYLOAD_NONE );
 				}
 
 				//
@@ -1268,7 +1194,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 
 				packet.done();
 
-				packet_ike_send( ph1, ph1, packet, true );
+				packet_ike_send( ph1, ph1, packet, false );
 
 				ph1->xstate |= XSTATE_SENT_SA;
 				ph1->xstate |= XSTATE_SENT_KE;
@@ -1317,7 +1243,6 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 					ph1->status( XCH_STATUS_DEAD, XCH_FAILED_PEER_AUTH, ISAKMP_N_AUTHENTICATION_FAILED );
 
 				ph1->clean();
-				ph1->resend_clear( true );
 			}
 		}
 
@@ -1337,7 +1262,6 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 					ph1->status( XCH_STATUS_DEAD, XCH_FAILED_PEER_AUTH, ISAKMP_N_AUTHENTICATION_FAILED );
 
 				ph1->clean();
-				ph1->resend_clear( true );
 			}
 		}
 	}
@@ -1423,7 +1347,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 
 				if( !ph1->initiator && ph1->vendopts_l.flag.xauth )
 				{
-					IDB_CFG * cfg = new IDB_CFG( ph1->tunnel, true, 0 );
+					IDB_CFG * cfg = new IDB_CFG( ph1, true );
 					cfg->add( true );
 					process_config_send( ph1, cfg );
 					cfg->dec( true );
@@ -1447,7 +1371,7 @@ long _IKED::process_phase1_send( IDB_PH1 * ph1 )
 
 					if( ph1->tunnel->peer->xconf_mode != CONFIG_MODE_PUSH )
 					{
-						IDB_CFG * cfg = new IDB_CFG( ph1->tunnel, true, 0 );
+						IDB_CFG * cfg = new IDB_CFG( ph1, true );
 						cfg->add( true );
 						process_config_send( ph1, cfg );
 						cfg->dec( true );
@@ -1550,6 +1474,38 @@ long _IKED::phase1_gen_keys( IDB_PH1 * ph1 )
 	IKE_PROPOSAL * proposal;
 	ph1->plist_l.get( &proposal, 0 );
 
+	//
+	// determine shared secret
+	//
+
+	if( level >= LLOG_DECODE )
+	{
+		BDATA prv;
+		prv.size( ph1->dh_size );
+		BN_bn2bin( ph1->dh->priv_key, prv.buff() );
+
+		log.bin(
+			LLOG_DECODE,
+			LLOG_DECODE,
+			prv.buff(),
+			prv.size(),
+			"ii : computed DH private key" );
+
+		log.bin(
+			LLOG_DECODE,
+			LLOG_DECODE,
+			ph1->xl.buff(),
+			ph1->xl.size(),
+			"ii : computed DH public key" );
+
+		log.bin(
+			LLOG_DECODE,
+			LLOG_DECODE,
+			ph1->xr.buff(),
+			ph1->xr.size(),
+			"ii : received DH public key" );
+	}
+
 	//
 	// validate the dh group size
 	//
@@ -1557,25 +1513,48 @@ long _IKED::phase1_gen_keys( IDB_PH1 * ph1 )
 	if( ph1->xr.size() != ph1->dh_size )
 	{
 		log.txt( LLOG_ERROR,
-			"!! : dh group size mismatch ( %d != %d )\n",
+			"!! : DH group size mismatch ( %d != %d )\n",
 			ph1->xr.size(),
 			ph1->dh_size );
 
+		ph1->status( XCH_STATUS_DEAD, XCH_FAILED_MSG_FORMAT, 0 );
 		return LIBIKE_FAILED;
 	}
 
 	//
-	// compute shared secret
+	// determine shared secret
 	//
 
 	BIGNUM * gx = BN_new();
 	BN_bin2bn( ph1->xr.buff(), ph1->dh_size, gx );
 
 	BDATA shared;
-	shared.size( ph1->dh_size );
-	DH_compute_key( shared.buff(), gx, ph1->dh );
+	shared.set( 0, ph1->dh_size );
+	long result = DH_compute_key( shared.buff(), gx, ph1->dh );
 	BN_free( gx );
 
+	if( result < 0 )
+	{
+		log.txt( LLOG_ERROR,
+			"!! : failed to compute DH shared secret\n" );
+
+		ph1->status( XCH_STATUS_DEAD, XCH_FAILED_MSG_CRYPTO, 0 );
+		return LIBIKE_FAILED;
+	}
+
+	//
+	// fixup shared secret buffer alignment
+	//
+
+	if( ph1->dh_size > result )
+	{
+		log.txt( LLOG_DEBUG,
+			"ww : short DH shared secret computed\n" );
+
+		shared.size( result );
+		shared.ins( 0, ph1->dh_size - result );
+	}
+
 	log.bin(
 		LLOG_DEBUG,
 		LLOG_DECODE,
@@ -1772,11 +1751,9 @@ long _IKED::phase1_gen_keys( IDB_PH1 * ph1 )
 
 		for( long size = skeyid_size; size < key_size; size += skeyid_size )
 		{
-			unsigned int temp;
-
 			HMAC_Init( &ctx_prf, skeyid_data, skeyid_size, ph1->evp_hash );
 			HMAC_Update( &ctx_prf, key_data + size - skeyid_size, skeyid_size );
-			HMAC_Final( &ctx_prf, key_data + size, &temp );
+			HMAC_Final( &ctx_prf, key_data + size, NULL );
 		}
 
 		HMAC_cleanup( &ctx_prf );
@@ -1960,15 +1937,15 @@ long _IKED::phase1_add_vend( IDB_PH1 * ph1, PACKET_IKE & packet, uint8_t next )
 	if( ph1->vendopts_l.flag.ssoft )
 		vid_count++;
 
-	if( ph1->vendopts_l.flag.unity )
-		vid_count++;
-
 	if( ph1->vendopts_l.flag.netsc )
 		vid_count++;
 
 	if( ph1->vendopts_l.flag.swind )
 		vid_count++;
 
+	if( ph1->vendopts_l.flag.unity )
+		vid_count++;
+
 	if( ph1->vendopts_l.flag.chkpt )
 		vid_count++;
 
@@ -2047,16 +2024,6 @@ long _IKED::phase1_add_vend( IDB_PH1 * ph1, PACKET_IKE & packet, uint8_t next )
 		log.txt( LLOG_INFO, "ii : local is SHREW SOFT compatible\n" );
 	}
 
-	//
-	// optionally add unity vendor id payload
-	//
-
-	if( ph1->vendopts_l.flag.unity )
-	{
-		payload_add_vend( packet, vend_unity, vendpld( vid_count, next ) );
-		log.txt( LLOG_INFO, "ii : local is CISCO UNITY compatible\n" );
-	}
-
 	//
 	// optionally add netscreen vendor payload
 	//
@@ -2077,6 +2044,21 @@ long _IKED::phase1_add_vend( IDB_PH1 * ph1, PACKET_IKE & packet, uint8_t next )
 		log.txt( LLOG_INFO, "ii : local is SIDEWINDER compatible\n" );
 	}
 
+	//
+	// optionally add unity vendor id payload
+	//
+
+	if( ph1->vendopts_l.flag.unity )
+	{
+		BDATA vend_unity2;
+		vend_unity2.add( vend_unity );	// base vendor id
+		vend_unity2.add( 0x02, 1 );		// major version
+		vend_unity2.add( 0x04, 1 );		// minor version
+
+		payload_add_vend( packet, vend_unity2, vendpld( vid_count, next ) );
+		log.txt( LLOG_INFO, "ii : local is CISCO UNITY compatible\n" );
+	}
+
 	//
 	// prepair and optionally add checkpoint vendor
 	// id payload ( must be last )
@@ -2301,17 +2283,6 @@ long _IKED::phase1_chk_vend( IDB_PH1 * ph1, BDATA & vend )
 		return LIBIKE_OK;
 	}
 
-	//
-	// check for unity vendor id
-	//
-
-	if( vendcmp( vend, vend_unity ) )
-	{
-		ph1->vendopts_r.flag.unity = true;
-		log.txt( LLOG_INFO, "ii : peer is CISCO UNITY compatible\n" );
-		return LIBIKE_OK;
-	}
-
 	//
 	// check for netscreen vendor id
 	//
@@ -2345,6 +2316,27 @@ long _IKED::phase1_chk_vend( IDB_PH1 * ph1, BDATA & vend )
 		return LIBIKE_OK;
 	}
 
+	//
+	// check for unity vendor id
+	//
+
+	if( vendcmp( vend, vend_unity, true ) )
+	{
+		//
+		// if we are communcating with a
+		// cisco unity device, set the
+		// policy init flag. this forces
+		// a single phase2 SA negotiation
+		// for the first policy created
+		// 
+
+		ph1->tunnel->tstate |= TSTATE_POLICY_INIT;
+
+		ph1->vendopts_r.flag.unity = true;
+		log.txt( LLOG_INFO, "ii : peer is CISCO UNITY compatible\n" );
+		return LIBIKE_OK;
+	}
+
 	//
 	// check for checkpoint vendor id
 	//
@@ -2484,44 +2476,69 @@ long _IKED::phase1_chk_sign( IDB_PH1 * ph1 )
 
 long _IKED::phase1_gen_natd( IDB_PH1 * ph1 )
 {
-	//
-	// compute the nat discovery
-	// hash for local address
-	//
+	if( ph1->lstate & LSTATE_GENNATD )
+		return LIBIKE_OK;
 
-	if( !ph1->natd_ls.size( ph1->hash_size ) )
+	BDATA natd;
+	if( !natd.size( ph1->hash_size ) )
 		return LIBIKE_MEMORY;
 
-	EVP_MD_CTX ctx_hash;
-	EVP_DigestInit( &ctx_hash, ph1->evp_hash );
-	EVP_DigestUpdate( &ctx_hash, ph1->cookies.i, ISAKMP_COOKIE_SIZE );
-	EVP_DigestUpdate( &ctx_hash, ph1->cookies.r, ISAKMP_COOKIE_SIZE );
-	EVP_DigestUpdate( &ctx_hash, &ph1->tunnel->saddr_l.saddr4.sin_addr.s_addr, 4 );
-	EVP_DigestUpdate( &ctx_hash, &ph1->tunnel->saddr_l.saddr4.sin_port, 2 );
-	EVP_DigestFinal( &ctx_hash, ph1->natd_ls.buff(), NULL );
-	EVP_MD_CTX_cleanup( &ctx_hash );
-
 	//
 	// compute the nat discovery
 	// hash for remote address
 	//
 
-	if( !ph1->natd_ld.size( ph1->hash_size ) )
-		return LIBIKE_MEMORY;
-
+	EVP_MD_CTX ctx_hash;
 	EVP_DigestInit( &ctx_hash, ph1->evp_hash );
 	EVP_DigestUpdate( &ctx_hash, ph1->cookies.i, ISAKMP_COOKIE_SIZE );
 	EVP_DigestUpdate( &ctx_hash, ph1->cookies.r, ISAKMP_COOKIE_SIZE );
 	EVP_DigestUpdate( &ctx_hash, &ph1->tunnel->saddr_r.saddr4.sin_addr.s_addr, 4 );
 	EVP_DigestUpdate( &ctx_hash, &ph1->tunnel->saddr_r.saddr4.sin_port, 2 );
-	EVP_DigestFinal( &ctx_hash, ph1->natd_ld.buff(), NULL );
+	EVP_DigestFinal( &ctx_hash, natd.buff(), NULL );
 	EVP_MD_CTX_cleanup( &ctx_hash );
 
+	ph1->natd_hash_l.add( natd );
+
+	//
+	// compute the nat discovery
+	// hash for local address
+	//
+
+	EVP_DigestInit( &ctx_hash, ph1->evp_hash );
+	EVP_DigestUpdate( &ctx_hash, ph1->cookies.i, ISAKMP_COOKIE_SIZE );
+	EVP_DigestUpdate( &ctx_hash, ph1->cookies.r, ISAKMP_COOKIE_SIZE );
+	EVP_DigestUpdate( &ctx_hash, &ph1->tunnel->saddr_l.saddr4.sin_addr.s_addr, 4 );
+	EVP_DigestUpdate( &ctx_hash, &ph1->tunnel->saddr_l.saddr4.sin_port, 2 );
+	EVP_DigestFinal( &ctx_hash, natd.buff(), NULL );
+	EVP_MD_CTX_cleanup( &ctx_hash );
+
+	ph1->natd_hash_l.add( natd );
+
 	ph1->lstate |= LSTATE_GENNATD;
 
 	return LIBIKE_OK;
 }
 
+bool _IKED::phase1_add_natd( IDB_PH1 * ph1, PACKET_IKE & packet, uint8_t next )
+{
+	BDATA natd;
+
+	long index = 0;
+	long count = ph1->natd_hash_l.count();
+
+	for( long index = 0; index < count; index++ )
+	{
+		ph1->natd_hash_l.get( natd, index );
+
+		if( index < ( count - 1 ) )
+			payload_add_natd( packet, natd, ph1->natt_pldtype );
+		else
+			payload_add_natd( packet, natd, next );
+	}
+
+	return true;
+}
+
 bool _IKED::phase1_chk_natd( IDB_PH1 * ph1 )
 {
 	//
@@ -2545,7 +2562,10 @@ bool _IKED::phase1_chk_natd( IDB_PH1 * ph1 )
 
 		case IPSEC_NATT_ENABLE:
 		{
-			bool xlated = false;
+			bool xlated_l = true;
+			bool xlated_r = true;
+			BDATA natd_l;
+			BDATA natd_r;
 
 			//
 			// make sure remote peer negotiated natt
@@ -2558,55 +2578,69 @@ bool _IKED::phase1_chk_natd( IDB_PH1 * ph1 )
 			}
 
 			//
-			// generate nat discovery if neccessary
+			// make sure local nat discovery hashes
+			// have been generated
 			//
 
-			if( !( ph1->lstate & LSTATE_GENNATD ) )
-				phase1_gen_natd( ph1 );
+			phase1_gen_natd( ph1 );
 
 			//
 			// compare the remote destination
-			// hash to the local source hash
+			// hash to all local source hashes
 			//
 
-			if( ph1->natd_rd.size() == ph1->natd_ls.size() )
+			if( !ph1->natd_hash_r.get( natd_r, 0 ) )
 			{
-				if( memcmp(
-						ph1->natd_rd.buff(),
-						ph1->natd_ls.buff(),
-						ph1->natd_ls.size() ) )
-				{
-					log.txt( LLOG_INFO,
-						"ii : nat discovery - local address is translated\n" );
+				log.txt( LLOG_ERROR,
+					"!! : no remote desitnation hash available for comparison\n" );
+				break;
+			}
 
-					xlated = true;
+			for( long index = 1; index < ph1->natd_hash_l.count(); index++ )
+			{
+				ph1->natd_hash_l.get( natd_l, index );
+				if( natd_r == natd_l )
+				{
+					xlated_l = false;
+					break;
 				}
 			}
 
+			if( xlated_l )
+				log.txt( LLOG_INFO,
+					"ii : nat discovery - local address is translated\n" );
+
 			//
-			// compare the remote source hash
-			// to the local destination hash
+			// compare the local destination
+			// hash to all remote source hashes
 			//
 
-			if( ph1->natd_rs.size() == ph1->natd_ld.size() )
+			if( !ph1->natd_hash_l.get( natd_l, 0 ) )
 			{
-				if( memcmp(
-						ph1->natd_rs.buff(),
-						ph1->natd_ld.buff(),
-						ph1->natd_ld.size() ) )
-				{
-					log.txt( LLOG_INFO,
-						"ii : nat discovery - remote address is translated\n" );
+				log.txt( LLOG_ERROR,
+					"!! : no local desitnation hash available for comparison\n" );
+				break;
+			}
 
-					xlated = true;
+			for( long index = 1; index < ph1->natd_hash_r.count(); index++ )
+			{
+				ph1->natd_hash_r.get( natd_r, index );
+				if( natd_l == natd_r )
+				{
+					xlated_r = false;
+					break;
 				}
 			}
 
+			if( xlated_r )
+				log.txt( LLOG_INFO,
+					"ii : nat discovery - remote address is translated\n" );
+
 			//
 			// only set the nat-t port if translation was detected
 			//
 
-			if( xlated )
+			if( xlated_l || xlated_r )
 			{
 				ph1->tunnel->natt_version = ph1->natt_version;
 				break;
@@ -2699,31 +2733,29 @@ bool _IKED::phase1_chk_port( IDB_PH1 * ph1, IKE_SADDR * saddr_r, IKE_SADDR * sad
 	if( saddr_r->saddr4.sin_port != ph1->tunnel->saddr_r.saddr4.sin_port )
 	{
 
-		if( !ph1->initiator )
-		{
-			log.txt( LLOG_ERROR,
-				"!! : responder port values have floated\n" );
-
-			return false;
-		}
-		else
+		if( ph1->initiator )
 		{
 			if( ph1->tunnel->peer->natt_mode == IPSEC_NATT_NONE )
 			{
 				log.txt( LLOG_INFO,
-					"ii : initiator port values floated but nat-t is disabled but \n" );
+					"ii : initiator port values floated but nat-t is disabled\n" );
 
 				return false;
 			}
 
 			if( ph1->tunnel->lstate & TSTATE_NATT_FLOAT )
 			{
-				log.txt( LLOG_ERROR,
-					"!! : initiator port values should only float once per session\n" );
+				log.txt( LLOG_INFO,
+					"ww : initiator port values should only float once per session\n" );
 
-				return false;
+				return true;
 			}
 		}
+		else
+		{
+			log.txt( LLOG_DEBUG,
+				"ii : responder port values have floated\n" );
+		}
 
 		//
 		// float our port to natt
