@@ -326,6 +326,10 @@
 
 #define IKE_FRAG_FLAG_LAST			0x01
 
+// cisco high availability
+
+#define ISAKMP_N_UNITY_LOAD_BALANCE	40501
+
 // dead peer detection
 
 #define ISAKMP_N_DPD_R_U_THERE		36136
@@ -482,6 +486,7 @@
 #define IPSEC_OPTS_DOMAIN		0x0080
 #define IPSEC_OPTS_SPLITDNS		0x0100
 #define IPSEC_OPTS_SAVEPW		0x0200
+#define IPSEC_OPTS_VEND_CHKPT	0x0400
 
 #define IPSEC_DNSS_MAX			4
 #define IPSEC_NBNS_MAX			2
@@ -749,8 +754,8 @@ typedef class _PACKET_IKE : public _PACKET
 
 	void	reset();
 
-	void	set_msgid( uint32_t msgid );
-	void	get_msgid( uint32_t & msgid );
+	void		set_msgid( uint32_t msgid );
+	uint32_t	get_msgid();
 
 	bool	add_payload( bool encap, uint8_t next );
 	bool	get_payload( bool encap, uint8_t & next );
