@@ -63,22 +63,6 @@ bool _ITH_EVENT_PH1SOFT::func()
 
 	if( ph1->tunnel->peer->contact == IPSEC_CONTACT_CLIENT )
 	{
-		if( ph1->vendopts_l.flag.xauth )
-		{
-			ph1->tunnel->tstate &= ~TSTATE_SENT_XAUTH;
-			ph1->tunnel->tstate &= ~TSTATE_RECV_XAUTH;
-			ph1->tunnel->tstate &= ~TSTATE_SENT_XRSLT;
-			ph1->tunnel->tstate &= ~TSTATE_RECV_XRSLT;
-		}
-
-		if( ph1->tunnel->peer->xconf_mode == CONFIG_MODE_PUSH )
-		{
-			ph1->tunnel->tstate &= ~TSTATE_RECV_CONFIG;
-			ph1->tunnel->tstate &= ~TSTATE_SENT_CONFIG;
-			ph1->tunnel->tstate &= ~TSTATE_RECV_ACK;
-			ph1->tunnel->tstate &= ~TSTATE_SENT_ACK;
-		}
-
 		IDB_PH1 * addph1 = new IDB_PH1( ph1->tunnel, true, NULL );
 		addph1->add( false );
 		iked.process_phase1_send( addph1 );
@@ -288,6 +272,7 @@ _IDB_PH1::_IDB_PH1( IDB_TUNNEL * set_tunnel, bool set_initiator, IKE_COOKIES * s
 {
 	evp_cipher = NULL;
 	evp_hash = NULL;
+	hash_size = 0;
 
 	memset( &cookies, 0, sizeof( cookies ) );
 
@@ -297,9 +282,6 @@ _IDB_PH1::_IDB_PH1( IDB_TUNNEL * set_tunnel, bool set_initiator, IKE_COOKIES * s
 	natt_version = IPSEC_NATT_NONE;
 	natt_pldtype = ISAKMP_PAYLOAD_NONE;
 
-	natted_l = false;
-	natted_r = false;
-
 	ctype_l = 0;
 	ctype_r = 0;
 
@@ -369,11 +351,13 @@ _IDB_PH1::_IDB_PH1( IDB_TUNNEL * set_tunnel, bool set_initiator, IKE_COOKIES * s
 
 	if( tunnel->peer->contact == IPSEC_CONTACT_CLIENT )
 	{
-		vendopts_l.flag.unity = true;
 		vendopts_l.flag.netsc = true;
 		vendopts_l.flag.zwall = true;
 		vendopts_l.flag.swind = true;
-		vendopts_l.flag.chkpt = true;
+		vendopts_l.flag.unity = true;
+
+		if( tunnel->xconf.opts & IPSEC_OPTS_VEND_CHKPT )
+			vendopts_l.flag.chkpt = true;
 	}
 
 	//
@@ -490,9 +474,9 @@ _IDB_PH1::~_IDB_PH1()
 // abstract functions from parent class
 //
 
-char * _IDB_PH1::name()
+const char * _IDB_PH1::name()
 {
-	static char * xname = "phase1";
+	static const char * xname = "phase1";
 	return xname;
 }
 
@@ -511,7 +495,7 @@ void _IDB_PH1::end()
 	// clear the resend queue
 	//
 
-	resend_clear( false );
+	resend_clear( false, true );
 
 	//
 	// remove scheduled events
@@ -550,6 +534,46 @@ void _IDB_PH1::end()
 		( xch_errorcode != XCH_FAILED_PEER_DELETE ) )
 		iked.inform_new_delete( this, NULL );
 
+	//
+	// if this sa has reached maturity,
+	// locate any config handles used by
+	// this sa and delete them
+	//
+	// FIXME : There must be a better way
+	//
+
+	if( lstate & LSTATE_HASKEYS )
+	{
+		//
+		// FIXME : Use find here
+		//
+
+		long cfg_count = iked.idb_list_cfg.count();
+		long cfg_index = 0;
+
+		for( ; cfg_index < cfg_count; cfg_index++ )
+		{
+			//
+			// get the next config in our list
+			// and attempt to match by pointer
+			// 
+
+			IDB_CFG * cfg = iked.idb_list_cfg.get( cfg_index );
+			if( cfg->ph1ref == this )
+			{
+				cfg->inc( false );
+
+				cfg->status( XCH_STATUS_DEAD, XCH_FAILED_PENDING, 0 );
+
+				if( cfg->dec( false ) )
+				{
+					cfg_index--;
+					cfg_count--;
+				}
+			}
+		}
+	}
+
 	//
 	// if this sa never reached maturity,
 	// locate any pending phase2 handles
@@ -644,12 +668,25 @@ bool _IDB_PH1::setup_dhgrp( IKE_PROPOSAL * proposal )
 	// initialize dh group
 	//
 
-	dh = dh_setup_group( proposal->dhgr_id );
-	dh_size = BN_num_bytes( dh->p );
-	dh_create_e( dh, dh_size );
+	if( !dh_init( proposal->dhgr_id, &dh, &dh_size ) )
+	{
+		iked.log.txt( LLOG_ERROR, "ii : failed to setup DH group\n" );
+		return false;
+	}
 
 	xl.size( dh_size );
-	BN_bn2bin( dh->pub_key, xl.buff() );
+	long result = BN_bn2bin( dh->pub_key, xl.buff() );
+
+	//
+	// fixup public buffer alignment
+	//
+
+	if( dh_size > result )
+	{
+		iked.log.txt( LLOG_DEBUG, "ww : short DH public value\n" );
+		xl.size( result );
+		xl.ins( 0, dh_size - result );
+	}
 
 	return true;
 }
@@ -765,10 +802,8 @@ void _IDB_PH1::clean()
 	idi.del( true );
 	idr.del( true );
 
-	natd_ls.del( true );
-	natd_ld.del( true );
-	natd_rs.del( true );
-	natd_rd.del( true );
+	natd_hash_l.clean();
+	natd_hash_r.clean();
 
 	sign_r.del( true );
 }
