@@ -69,9 +69,6 @@ long _IKED::loop_ike_pfkey()
 
 		long result = pfki.recv_message( msg );
 
-		if( result == IPCERR_NODATA )
-			continue;
-
 		if( result == IPCERR_CLOSED )
 		{
 			pfki.detach();
@@ -108,6 +105,9 @@ long _IKED::loop_ike_pfkey()
 			continue;
 		}
 
+		if( result != IPCERR_OK )
+			continue;
+
 		if( msg.header.sadb_msg_errno )
 		{
 			log.txt( LLOG_ERROR,
@@ -677,9 +677,9 @@ long _IKED::pfkey_recv_spadd( PFKI_MSG & msg )
 	policy->sp.id = spinfo.sp.id;
 
 	//
-	// if this policy was marked as
-	// nailed for a client tunnel,
-	// call init phase2 now
+	// if this policy was marked as nailed
+	// or initial for a client tunnel, call
+	// init phase2 now
 	//
 
 	if( policy->nailed )
@@ -688,6 +688,14 @@ long _IKED::pfkey_recv_spadd( PFKI_MSG & msg )
 		pfkey_init_phase2( true, spinfo.sp.type, spinfo.sp.id, 0 );
 	}
 
+	if( policy->initial )
+	{
+		policy->initial = false;
+
+		log.txt( LLOG_DEBUG, "ii : calling init phase2 for initial policy\n" );
+		pfkey_init_phase2( true, spinfo.sp.type, spinfo.sp.id, 0 );
+	}
+
 	policy->dec( true );
 
 	return LIBIKE_OK;
@@ -1023,8 +1031,6 @@ long _IKED::pfkey_recv_getspi( PFKI_MSG & msg )
 		log.txt( LLOG_DEBUG, "ii : waiting for %i spi updates\n", ph2->spicount );
 	else
 	{
-		ph2->lstate |= LSTATE_HASSPI;
-
 		IDB_PH1 * ph1 = NULL;
 
 		if( ph2->initiator )
