@@ -43,39 +43,36 @@
 
 long _IKED::payload_add_frag( PACKET_IKE & packet, unsigned char & index, unsigned char * data, size_t & size, size_t max )
 {
-	log.txt( LOG_DEBUG, ">> : fragment payload\n" );
+	log.txt( LLOG_DEBUG, ">> : fragment payload\n" );
 
 	//
-	// adjust the callers max to
-	// account for the fragment
-	// header size
+	// adjust the callers max to account for
+	// the fragment header size
 	//
 
 	max -= 8;
 
 	//
-	// calculate the source packets
-	// fragment offset, size and if
-	// this is the last fragment
+	// check if this will be the last fragment
+	// and calculate the source packets offset
+	// and size
 	//
 
+	unsigned char flags = 0;
+	if( size <= max )
+		flags |= IKE_FRAG_FLAG_LAST;
+
 	if( size > max )
 		size = max;
 
-	unsigned char flags = 0;
-	if( size < max )
-		flags |= IKE_FRAG_FLAG_LAST;
-
 	//
-	// sanity check size to make
-	// sure it doesn't overflow
-	// the 16bit value ( really
-	// should never happen )
+	// sanity check size to make sure it doesn't
+	// overflow the 16bit value
 	//
 
 	if( ( size + 8 ) > 0xFFFF )
 	{
-		log.txt( LOG_DEBUG, "!! : ike fragment length > 16bit\n" );
+		log.txt( LLOG_DEBUG, "!! : ike fragment length > 16bit\n" );
 		return LIBIKE_FAILED;
 	}
 
@@ -100,7 +97,7 @@ long _IKED::payload_add_frag( PACKET_IKE & packet, unsigned char & index, unsign
 
 long _IKED::payload_get_frag( PACKET_IKE & packet, IDB_PH1 * ph1, bool & complete )
 {
-	log.txt( LOG_DEBUG, "<< : fragment payload\n" );
+	log.txt( LLOG_DEBUG, "<< : fragment payload\n" );
 
 	//
 	// read the frag payload header
@@ -128,10 +125,10 @@ long _IKED::payload_get_frag( PACKET_IKE & packet, IDB_PH1 * ph1, bool & complet
 	// perform some sanity check
 	//
 
-	size_t data_size = packet.size() - packet.oset();
-	if( data_size < size )
+	size_t data_real = packet.size() - packet.oset();
+	if( data_real < size )
 	{
-		log.txt( LOG_ERROR, "!! : packet size is invalid for given fragment size\n" );
+		log.txt( LLOG_ERROR, "!! : packet size is invalid for given fragment size\n" );
 		return LIBIKE_DECODE;
 	}
 
@@ -226,13 +223,13 @@ long _IKED::payload_get_attr( PACKET_IKE & packet, IKE_ATTR & attrib )
 	return LIBIKE_OK;
 }
 
-long _IKED::payload_add_sa( PACKET_IKE & packet, IKE_PLIST & plist, uint8_t next )
+long _IKED::payload_add_sa( PACKET_IKE & packet, IDB_LIST_PROPOSAL & plist, uint8_t next )
 {
 	//
 	// write security association payload
 	//
 
-	log.txt( LOG_DEBUG, ">> : security association payload\n" );
+	log.txt( LLOG_DEBUG, ">> : security association payload\n" );
 
 	packet.add_payload( false, next );						// ADD - sa
 	packet.add_quad( ISAKMP_DOI_IPSEC );					// domain of interop
@@ -254,7 +251,7 @@ long _IKED::payload_add_sa( PACKET_IKE & packet, IKE_PLIST & plist, uint8_t next
 		// write proposal payload
 		//
 
-		log.txt( LOG_DEBUG, ">> : - proposal #%i payload \n", proposal->pnumb );
+		log.txt( LLOG_DEBUG, ">> : - proposal #%i payload \n", proposal->pnumb );
 
 		next = ISAKMP_PAYLOAD_PROPOSAL;
 		if( pindex == -1 )
@@ -319,9 +316,9 @@ long _IKED::payload_add_sa( PACKET_IKE & packet, IKE_PLIST & plist, uint8_t next
 	return LIBIKE_OK;
 }
 
-long _IKED::payload_get_sa( PACKET_IKE & packet, IKE_PLIST & plist )
+long _IKED::payload_get_sa( PACKET_IKE & packet, IDB_LIST_PROPOSAL & plist )
 {
-	log.txt( LOG_DEBUG, "<< : security association payload\n" );
+	log.txt( LLOG_DEBUG, "<< : security association payload\n" );
 
 	//
 	// read security association packet
@@ -338,7 +335,7 @@ long _IKED::payload_get_sa( PACKET_IKE & packet, IKE_PLIST & plist )
 
 	if( doi != ISAKMP_DOI_IPSEC )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"!! : rejecting sa payload, invalid doi ( %i )\n",
 			doi );
 
@@ -355,7 +352,7 @@ long _IKED::payload_get_sa( PACKET_IKE & packet, IKE_PLIST & plist )
 
 	if( doi != ISAKMP_SIT_IDENT_ONLY )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"!! : rejecting sa payload, invalid situation ( %i )\n",
 			sit );
 
@@ -390,7 +387,7 @@ long _IKED::payload_get_sa( PACKET_IKE & packet, IKE_PLIST & plist )
 		packet.get_byte( spi.size );
 		packet.get_byte( xform_count );
 
-		log.txt( LOG_DEBUG, "<< : - propsal #%i payload \n", pnumb );
+		log.txt( LLOG_DEBUG, "<< : - propsal #%i payload \n", pnumb );
 
 		//
 		// validate the protocol and spi
@@ -402,7 +399,7 @@ long _IKED::payload_get_sa( PACKET_IKE & packet, IKE_PLIST & plist )
 		{
 			case ISAKMP_PROTO_ISAKMP:
 
-				if( spi.size && ( spi.size != ISAKMP_COOKIE_SIZE * 2 ) )
+				if( spi.size % ISAKMP_COOKIE_SIZE )
 					badspi = true;
 				else
 					packet.get_null( spi.size );
@@ -430,7 +427,7 @@ long _IKED::payload_get_sa( PACKET_IKE & packet, IKE_PLIST & plist )
 
 			default:
 
-				log.txt( LOG_ERROR,
+				log.txt( LLOG_ERROR,
 					"\n"
 					"!! : rejecting sa payload\n"
 					"!! : invalid protocol %s ( %i ) \n"
@@ -445,7 +442,7 @@ long _IKED::payload_get_sa( PACKET_IKE & packet, IKE_PLIST & plist )
 
 		if( badspi )
 		{
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : invalid spi size of %i for protocol %s \n",
 				spi.size,
 				find_name( NAME_PROTOCOL, proto ) );
@@ -502,7 +499,7 @@ long _IKED::payload_add_xform( PACKET_IKE & packet, IKE_PROPOSAL * proposal, uin
 	// write transform payload
 	//
 
-	log.txt( LOG_DEBUG, ">> : -- transform #%i payload \n", proposal->tnumb );
+	log.txt( LLOG_DEBUG, ">> : -- transform #%i payload \n", proposal->tnumb );
 
 	packet.add_payload( true, next );						// ADD -> sa.proposal.transform
 	packet.add_byte( proposal->tnumb );						// transform number
@@ -626,7 +623,7 @@ long _IKED::payload_add_xform( PACKET_IKE & packet, IKE_PROPOSAL * proposal, uin
 
 		default:
 
-			log.txt( LOG_ERROR, "!! : invalid protocol in proposal" );
+			log.txt( LLOG_ERROR, "!! : invalid protocol in proposal" );
 
 			return LIBIKE_FAILED;
 	}
@@ -651,7 +648,7 @@ long _IKED::payload_get_xform( PACKET_IKE & packet, IKE_PROPOSAL * proposal )
 	packet.get_byte( proposal->xform );		// transform type
 	packet.get_word( reserved );			// reserved 2 bytes
 
-	log.txt( LOG_DEBUG, "<< : -- transform #%i payload \n", proposal->tnumb );
+	log.txt( LLOG_DEBUG, "<< : -- transform #%i payload \n", proposal->tnumb );
 
 	//
 	// validate the transform type
@@ -700,8 +697,7 @@ long _IKED::payload_get_xform( PACKET_IKE & packet, IKE_PROPOSAL * proposal )
 			break;
 	}
 
-	size_t size;
-	packet.chk_payload( size );
+	size_t size = packet.get_payload_left();
 
 	uint32_t lvalue = 0;
 	uint16_t svalue = 0;
@@ -838,7 +834,7 @@ long _IKED::payload_get_xform( PACKET_IKE & packet, IKE_PROPOSAL * proposal )
 
 							default:
 
-								log.txt( LOG_ERROR,
+								log.txt( LLOG_ERROR,
 									"\n"
 									"!! : rejecting phase1 proposal\n"
 									"!! : unhandled life type ( %i )\n"
@@ -855,7 +851,7 @@ long _IKED::payload_get_xform( PACKET_IKE & packet, IKE_PROPOSAL * proposal )
 
 					default:
 					{
-						log.txt( LOG_ERROR,
+						log.txt( LLOG_ERROR,
 							"\n"
 							"!! : rejecting phase1 proposal\n"
 							"!! : unhandled attribute type ( %i )\n"
@@ -909,7 +905,7 @@ long _IKED::payload_get_xform( PACKET_IKE & packet, IKE_PROPOSAL * proposal )
 
 							default:
 
-								log.txt( LOG_ERROR,
+								log.txt( LLOG_ERROR,
 									"\n"
 									"!! : rejecting phase2 proposal\n"
 									"!! : unhandled life type ( %i )\n"
@@ -950,7 +946,7 @@ long _IKED::payload_get_xform( PACKET_IKE & packet, IKE_PROPOSAL * proposal )
 
 					default:
 					{
-						log.txt( LOG_ERROR,
+						log.txt( LLOG_ERROR,
 							"\n"
 							"!! : rejecting phase2 proposal\n"
 							"!! : unhandled attribute type ( %i )\n"
@@ -967,7 +963,7 @@ long _IKED::payload_get_xform( PACKET_IKE & packet, IKE_PROPOSAL * proposal )
 			}
 		}
 
-		packet.chk_payload( size );
+		size = packet.get_payload_left();
 	}
 
 	//
@@ -981,7 +977,7 @@ long _IKED::payload_get_xform( PACKET_IKE & packet, IKE_PROPOSAL * proposal )
 
 long _IKED::payload_add_kex( PACKET_IKE & packet, BDATA & gx, uint8_t next )
 {
-	log.txt( LOG_DEBUG, ">> : key exchange payload\n" );
+	log.txt( LLOG_DEBUG, ">> : key exchange payload\n" );
 
 	//
 	// write key exchange payload
@@ -996,18 +992,17 @@ long _IKED::payload_add_kex( PACKET_IKE & packet, BDATA & gx, uint8_t next )
 
 long _IKED::payload_get_kex( PACKET_IKE & packet, BDATA & gx )
 {
-	log.txt( LOG_DEBUG, "<< : key exchange payload\n" );
+	log.txt( LLOG_DEBUG, "<< : key exchange payload\n" );
 
 	//
 	// read key exchange payload
 	//
 
-	size_t size;
-	packet.chk_payload( size );
+	size_t size = packet.get_payload_left();
 
 	if( size > LIBIKE_MAX_DHGRP )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"<< : invalid dh size ( %i > %i )\n",
 			size,
 			LIBIKE_MAX_DHGRP );
@@ -1024,7 +1019,7 @@ long _IKED::payload_get_kex( PACKET_IKE & packet, BDATA & gx )
 
 long _IKED::payload_add_nonce( PACKET_IKE & packet, BDATA & nonce, uint8_t next )
 {
-	log.txt( LOG_DEBUG, ">> : nonce payload\n" );
+	log.txt( LLOG_DEBUG, ">> : nonce payload\n" );
 
 	//
 	// write nonce payload
@@ -1039,14 +1034,13 @@ long _IKED::payload_add_nonce( PACKET_IKE & packet, BDATA & nonce, uint8_t next
 
 long _IKED::payload_get_nonce( PACKET_IKE & packet, BDATA & nonce )
 {
-	log.txt( LOG_DEBUG, "<< : nonce payload\n" );
+	log.txt( LLOG_DEBUG, "<< : nonce payload\n" );
 
 	//
 	// read nonce payload
 	//
 
-	size_t size;
-	packet.chk_payload( size );
+	size_t size = packet.get_payload_left();
 
 	if( ( size < ISAKMP_NONCE_MIN ) ||
 		( size > ISAKMP_NONCE_MAX ) )
@@ -1059,7 +1053,7 @@ long _IKED::payload_get_nonce( PACKET_IKE & packet, BDATA & nonce )
 
 long _IKED::payload_add_ph1id( PACKET_IKE & packet, IKE_PH1ID & ph1id, uint8_t next )
 {
-	log.txt( LOG_DEBUG, ">> : identification payload\n" );
+	log.txt( LLOG_DEBUG, ">> : identification payload\n" );
 
 	//
 	// write identity payload
@@ -1098,7 +1092,7 @@ long _IKED::payload_add_ph1id( PACKET_IKE & packet, IKE_PH1ID & ph1id, uint8_t n
 
 		default:
 
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : unhandled phase1 id type \'%s\'( %i )\n",
 				find_name( NAME_IDENT, ph1id.type ),
 				ph1id.type );
@@ -1113,7 +1107,7 @@ long _IKED::payload_add_ph1id( PACKET_IKE & packet, IKE_PH1ID & ph1id, uint8_t n
 
 long _IKED::payload_get_ph1id( PACKET_IKE & packet, IKE_PH1ID & ph1id )
 {
-	log.txt( LOG_DEBUG, "<< : identification payload\n" );
+	log.txt( LLOG_DEBUG, "<< : identification payload\n" );
 
 	//
 	// read responder identity payload
@@ -1145,7 +1139,7 @@ long _IKED::payload_get_ph1id( PACKET_IKE & packet, IKE_PH1ID & ph1id )
 		case ISAKMP_ID_ASN1_GN:
 		case ISAKMP_ID_KEY_ID:
 
-			packet.chk_payload( size );
+			size = packet.get_payload_left();
 			if( size < LIBIKE_MAX_VARID )
 				packet.get( ph1id.varid, size );		// string ids
 
@@ -1159,7 +1153,7 @@ long _IKED::payload_get_ph1id( PACKET_IKE & packet, IKE_PH1ID & ph1id )
 
 		default:
 
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : unhandled phase1 id type \'%s\'( %i )\n",
 				find_name( NAME_IDENT, ph1id.type ),
 				ph1id.type );
@@ -1176,7 +1170,7 @@ long _IKED::payload_get_ph1id( PACKET_IKE & packet, IKE_PH1ID & ph1id )
 
 long _IKED::payload_add_ph2id( PACKET_IKE & packet, IKE_PH2ID & ph2id, uint8_t next )
 {
-	log.txt( LOG_DEBUG, ">> : identification payload\n" );
+	log.txt( LLOG_DEBUG, ">> : identification payload\n" );
 
 	//
 	// write identity payload
@@ -1184,8 +1178,8 @@ long _IKED::payload_add_ph2id( PACKET_IKE & packet, IKE_PH2ID & ph2id, uint8_t n
 
 	packet.add_payload( false, next );		// ADD - identity
 	packet.add_byte( ph2id.type );			// id type
-	packet.add_byte( 0 );					// protocol ( ignore )
-	packet.add_word( 0 );					// IP port ( ignore )
+	packet.add_byte( ph2id.prot );			// protocol
+	packet.add_word( ph2id.port, false );	// IP port
 
 	switch( ph2id.type )
 	{
@@ -1205,7 +1199,7 @@ long _IKED::payload_add_ph2id( PACKET_IKE & packet, IKE_PH2ID & ph2id, uint8_t n
 
 		default:
 
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : unhandled ipv4 id type \'%s\'( %i )\n",
 				find_name( NAME_IDENT, ph2id.type ),
 				ph2id.type );
@@ -1220,20 +1214,18 @@ long _IKED::payload_add_ph2id( PACKET_IKE & packet, IKE_PH2ID & ph2id, uint8_t n
 
 long _IKED::payload_get_ph2id( PACKET_IKE & packet, IKE_PH2ID & ph2id )
 {
-	log.txt( LOG_DEBUG, "<< : identification payload\n" );
+	log.txt( LLOG_DEBUG, "<< : identification payload\n" );
 
 	//
 	// read responder identity payload
 	//
 
-	uint8_t		r_prot;
-	uint16_t	r_port;
 	uint32_t	temp1 = 0;
 	uint32_t	temp2 = 0;
 
 	packet.get_byte( ph2id.type );			// id type
-	packet.get_byte( r_prot );				// protocol ( ignore )
-	packet.get_word( r_port );				// IP port ( ignore )
+	packet.get_byte( ph2id.prot );			// protocol
+	packet.get_word( ph2id.port, false );	// IP port
 
 	switch( ph2id.type )
 	{
@@ -1253,7 +1245,7 @@ long _IKED::payload_get_ph2id( PACKET_IKE & packet, IKE_PH2ID & ph2id )
 
 		default:
 
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : unhandled ipv4 id type \'%s\'( %i )\n",
 				find_name( NAME_IDENT, ph2id.type ),
 				ph2id.type );
@@ -1271,7 +1263,7 @@ long _IKED::payload_get_ph2id( PACKET_IKE & packet, IKE_PH2ID & ph2id )
 
 long _IKED::payload_add_cert( PACKET_IKE & packet, uint8_t type, BDATA & cert, uint8_t next )
 {
-	log.txt( LOG_DEBUG, ">> : certificate payload\n" );
+	log.txt( LLOG_DEBUG, ">> : certificate payload\n" );
 
 	//
 	// write certificate payload
@@ -1287,19 +1279,18 @@ long _IKED::payload_add_cert( PACKET_IKE & packet, uint8_t type, BDATA & cert, u
 
 long _IKED::payload_get_cert( PACKET_IKE & packet, uint8_t & type, BDATA & cert )
 {
-	log.txt( LOG_DEBUG, "<< : certificate payload\n" );
+	log.txt( LLOG_DEBUG, "<< : certificate payload\n" );
 
 	//
 	// read certificate payload
 	//
 
-	size_t size;
-	packet.chk_payload( size );
+	size_t size = packet.get_payload_left();
 	size--;
 
 	if( size > ISAKMP_CERT_MAX )
 	{
-		log.txt( LOG_ERROR, "!! : invalid certificate size ( %i bytes )\n", size );
+		log.txt( LLOG_ERROR, "!! : invalid certificate size ( %i bytes )\n", size );
 
 		packet.notify = ISAKMP_N_INVALID_CERT_ENCODING;
 
@@ -1325,7 +1316,7 @@ long _IKED::payload_get_cert( PACKET_IKE & packet, uint8_t & type, BDATA & cert
 
 long _IKED::payload_add_creq( PACKET_IKE & packet, uint8_t type, uint8_t next )
 {
-	log.txt( LOG_DEBUG, ">> : cert request payload\n" );
+	log.txt( LLOG_DEBUG, ">> : cert request payload\n" );
 
 	//
 	// write certificate request payload
@@ -1340,7 +1331,7 @@ long _IKED::payload_add_creq( PACKET_IKE & packet, uint8_t type, uint8_t next )
 
 long _IKED::payload_get_creq( PACKET_IKE & packet, uint8_t & type )
 {
-	log.txt( LOG_DEBUG, "<< : cert request payload\n" );
+	log.txt( LLOG_DEBUG, "<< : cert request payload\n" );
 
 	//
 	// read certificate request payload
@@ -1348,19 +1339,18 @@ long _IKED::payload_get_creq( PACKET_IKE & packet, uint8_t & type )
 
 	if( !packet.get_byte( type ) )		// certificate request type
 	{
-		log.txt( LOG_ERROR, "!! : no certificate request type\n" );
+		log.txt( LLOG_ERROR, "!! : no certificate request type\n" );
 
 		packet.notify = ISAKMP_N_INVALID_COOKIE;
 
 		return LIBIKE_DECODE;
 	}
 
-	size_t size;
-	packet.chk_payload( size );
+	size_t size = packet.get_payload_left();
 
 	if( size > ISAKMP_CREQ_MAX )
 	{
-		log.txt( LOG_ERROR, "!! : invalid cert authority payload size\n" );
+		log.txt( LLOG_ERROR, "!! : invalid cert authority payload size\n" );
 
 		return LIBIKE_DECODE;
 	}
@@ -1383,7 +1373,7 @@ long _IKED::payload_get_creq( PACKET_IKE & packet, uint8_t & type )
 
 long _IKED::payload_add_sign( PACKET_IKE & packet, BDATA & sign, uint8_t next )
 {
-	log.txt( LOG_DEBUG, ">> : signature payload\n" );
+	log.txt( LLOG_DEBUG, ">> : signature payload\n" );
 
 	//
 	// write signature payload
@@ -1398,18 +1388,17 @@ long _IKED::payload_add_sign( PACKET_IKE & packet, BDATA & sign, uint8_t next )
 
 long _IKED::payload_get_sign( PACKET_IKE & packet, BDATA & sign )
 {
-	log.txt( LOG_DEBUG, "<< : signature payload\n" );
+	log.txt( LLOG_DEBUG, "<< : signature payload\n" );
 
 	//
 	// read signature payload
 	//
 
-	size_t size;
-	packet.chk_payload( size );
+	size_t size = packet.get_payload_left();
 
 	if( size > ISAKMP_SIGN_MAX )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"!! : invalid signature size ( %i > %i )\n",
 			size,
 			ISAKMP_SIGN_MAX );
@@ -1426,7 +1415,7 @@ long _IKED::payload_get_sign( PACKET_IKE & packet, BDATA & sign )
 
 long _IKED::payload_add_hash( PACKET_IKE & packet, BDATA & hash, uint8_t next )
 {
-	log.txt( LOG_DEBUG, ">> : hash payload\n" );
+	log.txt( LLOG_DEBUG, ">> : hash payload\n" );
 
 	//
 	// write hash payload
@@ -1441,18 +1430,17 @@ long _IKED::payload_add_hash( PACKET_IKE & packet, BDATA & hash, uint8_t next )
 
 long _IKED::payload_get_hash( PACKET_IKE & packet, BDATA & hash, long hash_size )
 {
-	log.txt( LOG_DEBUG, "<< : hash payload\n" );
+	log.txt( LLOG_DEBUG, "<< : hash payload\n" );
 
 	//
 	// read hash payload
 	//
 
-	size_t size;
-	packet.chk_payload( size );
+	size_t size = packet.get_payload_left();
 
 	if( size != hash_size )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"!! : invalid hash size ( %i != %i )\n",
 			size,
 			hash_size );
@@ -1469,7 +1457,7 @@ long _IKED::payload_get_hash( PACKET_IKE & packet, BDATA & hash, long hash_size
 
 long _IKED::payload_add_vend( PACKET_IKE & packet, BDATA & vend, uint8_t next )
 {
-	log.txt( LOG_DEBUG, ">> : vendor id payload\n" );
+	log.txt( LLOG_DEBUG, ">> : vendor id payload\n" );
 
 	//
 	// write vendor id payload
@@ -1484,14 +1472,13 @@ long _IKED::payload_add_vend( PACKET_IKE & packet, BDATA & vend, uint8_t next )
 
 long _IKED::payload_get_vend( PACKET_IKE & packet, BDATA & vend )
 {
-	log.txt( LOG_DEBUG, "<< : vendor id payload\n" );
+	log.txt( LLOG_DEBUG, "<< : vendor id payload\n" );
 
 	//
 	// read vendor id payload
 	//
 
-	size_t size;
-	packet.chk_payload( size );
+	size_t size = packet.get_payload_left();
 
 	//
 	// if the vendor id is greater
@@ -1510,7 +1497,7 @@ long _IKED::payload_get_vend( PACKET_IKE & packet, BDATA & vend )
 
 long _IKED::payload_add_cfglist( PACKET_IKE & packet, IDB_CFG * cfg, uint8_t next )
 {
-	log.txt( LOG_DEBUG, ">> : attribute payload\n" );
+	log.txt( LLOG_DEBUG, ">> : attribute payload\n" );
 
 	//
 	// write attribute payload
@@ -1556,7 +1543,7 @@ long _IKED::payload_add_cfglist( PACKET_IKE & packet, IDB_CFG * cfg, uint8_t nex
 
 long _IKED::payload_get_cfglist( PACKET_IKE & packet, IDB_CFG * cfg )
 {
-	log.txt( LOG_DEBUG, "<< : attribute payload\n" );
+	log.txt( LLOG_DEBUG, "<< : attribute payload\n" );
 
 	//
 	// read attribute payload
@@ -1570,8 +1557,7 @@ long _IKED::payload_get_cfglist( PACKET_IKE & packet, IDB_CFG * cfg )
 	// get remaining payload length
 	//
 
-	size_t size;
-	packet.chk_payload( size );
+	size_t size = packet.get_payload_left();
 
 	while( size )
 	{
@@ -1618,7 +1604,7 @@ long _IKED::payload_get_cfglist( PACKET_IKE & packet, IDB_CFG * cfg )
 		// get remaining payload length
 		//
 
-		packet.chk_payload( size );
+		size = packet.get_payload_left();
 	}
 
 	return LIBIKE_OK;
@@ -1626,7 +1612,7 @@ long _IKED::payload_get_cfglist( PACKET_IKE & packet, IDB_CFG * cfg )
 
 long _IKED::payload_add_natd( PACKET_IKE & packet, BDATA & natd, uint8_t next )
 {
-	log.txt( LOG_DEBUG, ">> : nat discovery payload\n" );
+	log.txt( LLOG_DEBUG, ">> : nat discovery payload\n" );
 
 	//
 	// write natd hash payload
@@ -1641,18 +1627,17 @@ long _IKED::payload_add_natd( PACKET_IKE & packet, BDATA & natd, uint8_t next )
 
 long _IKED::payload_get_natd( PACKET_IKE & packet, BDATA & natd, long natd_size )
 {
-	log.txt( LOG_DEBUG, "<< : nat discovery payload\n" );
+	log.txt( LLOG_DEBUG, "<< : nat discovery payload\n" );
 
 	//
 	// read hash payload
 	//
 
-	size_t size;
-	packet.chk_payload( size );
+	size_t size = packet.get_payload_left();
 
 	if( size != natd_size )
 	{
-		log.txt( LOG_ERROR,
+		log.txt( LLOG_ERROR,
 			"!! : invalid natd hash size ( %i != %i )\n",
 			size,
 			natd_size );
@@ -1669,7 +1654,7 @@ long _IKED::payload_get_natd( PACKET_IKE & packet, BDATA & natd, long natd_size
 
 long _IKED::payload_add_notify( PACKET_IKE & packet, IKE_NOTIFY * notify, uint8_t next )
 {
-	log.txt( LOG_DEBUG, ">> : notification payload\n" );
+	log.txt( LLOG_DEBUG, ">> : notification payload\n" );
 
 	//
 	// write notification payload
@@ -1710,7 +1695,7 @@ long _IKED::payload_add_notify( PACKET_IKE & packet, IKE_NOTIFY * notify, uint8_
 
 long _IKED::payload_get_notify( PACKET_IKE & packet, IKE_NOTIFY * notify )
 {
-	log.txt( LOG_DEBUG, "<< : notification payload\n" );
+	log.txt( LLOG_DEBUG, "<< : notification payload\n" );
 
 	//
 	// read notification payload
@@ -1746,7 +1731,7 @@ long _IKED::payload_get_notify( PACKET_IKE & packet, IKE_NOTIFY * notify )
 			break;
 
 		default:
-			log.txt( LOG_ERROR, "<< : notification payload contained invalid spi\n" );
+			log.txt( LLOG_ERROR, "<< : notification payload contained invalid spi\n" );
 			return LIBIKE_FAILED;
 	}
 
@@ -1754,8 +1739,7 @@ long _IKED::payload_get_notify( PACKET_IKE & packet, IKE_NOTIFY * notify )
 	// read any extra notify data
 	//
 
-	size_t size;
-	packet.chk_payload( size );
+	size_t size = packet.get_payload_left();
 	packet.get( notify->data, size );
 
 	return LIBIKE_OK;
@@ -1763,7 +1747,7 @@ long _IKED::payload_get_notify( PACKET_IKE & packet, IKE_NOTIFY * notify )
 
 long _IKED::payload_add_delete( PACKET_IKE & packet, IKE_NOTIFY * notify, uint8_t next )
 {
-	log.txt( LOG_DEBUG, ">> : delete payload\n" );
+	log.txt( LLOG_DEBUG, ">> : delete payload\n" );
 
 	//
 	// write delete payload
@@ -1798,7 +1782,7 @@ long _IKED::payload_add_delete( PACKET_IKE & packet, IKE_NOTIFY * notify, uint8_
 
 long _IKED::payload_get_delete( PACKET_IKE & packet, IKE_NOTIFY * notify )
 {
-	log.txt( LOG_DEBUG, "<< : delete payload\n" );
+	log.txt( LLOG_DEBUG, "<< : delete payload\n" );
 
 	//
 	// read notification payload
@@ -1837,7 +1821,7 @@ long _IKED::payload_get_delete( PACKET_IKE & packet, IKE_NOTIFY * notify )
 			break;
 
 		default:
-			log.txt( LOG_ERROR, "<< : notification payload contained invalid spi\n" );
+			log.txt( LLOG_ERROR, "<< : notification payload contained invalid spi\n" );
 			return LIBIKE_FAILED;
 	}
 
