@@ -43,10 +43,6 @@
 
 bool _IKED::gen_ph1id_l( IDB_PH1 * ph1, IKE_PH1ID & ph1id )
 {
-	//
-	// initialize local id
-	//
-
 	ph1id.type = ph1->tunnel->peer->idtype_l;
 
 	switch( ph1id.type )
@@ -62,16 +58,14 @@ bool _IKED::gen_ph1id_l( IDB_PH1 * ph1, IKE_PH1ID & ph1id )
 						ph1->tunnel->peer->iddata_l,
 						ph1id.varid ) )
 				{
-					char temp[ 512 ];
-					size_t size = ph1->tunnel->peer->iddata_l.size();
-					if( size > 511 )
-						size = 511;
-
-					memcpy( temp, ph1->tunnel->peer->iddata_l.buff(), size );
-					temp[ size ] = 0;
+					BDATA temp;
+					temp = ph1->tunnel->peer->iddata_l;
+					temp.add( "", 1 );
 
 					log.txt( LLOG_ERROR,
-						"!! : gen_ph1id_l failed. invalid initiator id data \'%s\'\n", temp );
+						"!! : failed to generate local %s id from \'%s\'\n",
+							find_name( NAME_IDENT, ph1id.type ),
+							temp.text() );
 
 					return false;
 				}
@@ -90,8 +84,21 @@ bool _IKED::gen_ph1id_l( IDB_PH1 * ph1, IKE_PH1ID & ph1id )
 		{
 			if( ph1->tunnel->peer->iddata_l.size() )
 			{
-				ph1id.addr.s_addr =
-					inet_addr( ( char * ) ph1->tunnel->peer->iddata_l.buff() );
+				BDATA temp;
+				temp = ph1->tunnel->peer->iddata_l;
+				temp.add( "", 1 );
+
+				ph1id.addr.s_addr = inet_addr( temp.text() );
+
+				if( ph1id.addr.s_addr == INADDR_NONE )
+				{
+					log.txt( LLOG_ERROR,
+						"!! : failed to generate local %s id from \'%s\'\n",
+							find_name( NAME_IDENT, ph1id.type ),
+							temp.text() );
+
+					return false;
+				}
 			}
 			else
 			{
@@ -114,7 +121,8 @@ bool _IKED::gen_ph1id_l( IDB_PH1 * ph1, IKE_PH1ID & ph1id )
 		default:
 		{
 			log.txt( LLOG_ERROR,
-				"!! : gen_ph1id_l failed. unhandled id type %i\n", ph1id.type );
+				"!! : failed to generate local id for unknown type %i\n",
+				ph1id.type );
 
 			return false;
 		}
@@ -137,16 +145,14 @@ bool _IKED::gen_ph1id_r( IDB_PH1 * ph1, IKE_PH1ID & ph1id )
 						ph1->tunnel->peer->iddata_r,
 						ph1id.varid ) )
 				{
-					char temp[ 512 ];
-					size_t size = ph1->tunnel->peer->iddata_r.size();
-					if( size > 511 )
-						size = 511;
-
-					memcpy( temp, ph1->tunnel->peer->iddata_r.buff(), size );
-					temp[ size ] = 0;
+					BDATA temp;
+					temp = ph1->tunnel->peer->iddata_r;
+					temp.add( "", 1 );
 
 					log.txt( LLOG_ERROR,
-						"!! : gen_ph1id failed. invalid responder id data \'%s\'\n", temp );
+						"!! : failed to generate remote %s id from \'%s\'\n",
+							find_name( NAME_IDENT, ph1id.type ),
+							temp.text() );
 
 					return false;
 				}
@@ -159,8 +165,21 @@ bool _IKED::gen_ph1id_r( IDB_PH1 * ph1, IKE_PH1ID & ph1id )
 		{
 			if( ph1->tunnel->peer->iddata_r.size() )
 			{
-				ph1id.addr.s_addr =
-					inet_addr( ( char * ) ph1->tunnel->peer->iddata_r.buff() );
+				BDATA temp;
+				temp = ph1->tunnel->peer->iddata_r;
+				temp.add( "", 1 );
+
+				ph1id.addr.s_addr = inet_addr( temp.text() );
+
+				if( ph1id.addr.s_addr == INADDR_NONE )
+				{
+					log.txt( LLOG_ERROR,
+						"!! : failed to generate remote %s id from \'%s\'\n",
+							find_name( NAME_IDENT, ph1id.type ),
+							temp.text() );
+
+					return false;
+				}
 			}
 			else
 			{
@@ -183,7 +202,8 @@ bool _IKED::gen_ph1id_r( IDB_PH1 * ph1, IKE_PH1ID & ph1id )
 		default:
 		{
 			log.txt( LLOG_ERROR,
-				"!! : gen_ph1id_r failed. unhandled id type %i\n", ph1id.type );
+				"!! : failed to generate remote id for unknown type %i\n",
+				ph1id.type );
 
 			return false;
 		}
@@ -194,6 +214,11 @@ bool _IKED::gen_ph1id_r( IDB_PH1 * ph1, IKE_PH1ID & ph1id )
 
 bool _IKED::cmp_ph1id( IKE_PH1ID & idt, IKE_PH1ID & ids, bool natt )
 {
+	static const char * expl_natt = "( natt prevents ip match )";
+	static const char * expl_cert = "( cert check only )";
+	static const char * expl_none = "";
+	const char * expl = expl_none;
+
 	//
 	// compare the peer id received
 	// with our generated peer id
@@ -202,7 +227,7 @@ bool _IKED::cmp_ph1id( IKE_PH1ID & idt, IKE_PH1ID & ids, bool natt )
 	if( ids.type != idt.type )
 	{
 		log.txt( LLOG_ERROR, 
-			"!! : phase1 id type mismatch ( %s != %s )\n",
+			"!! : phase1 id type mismatch ( received %s but expected %s )\n",
 			find_name( NAME_IDENT, ids.type ),
 			find_name( NAME_IDENT, idt.type ) );
 
@@ -223,15 +248,10 @@ bool _IKED::cmp_ph1id( IKE_PH1ID & idt, IKE_PH1ID & ids, bool natt )
 		case ISAKMP_ID_IPV4_ADDR:
 		{
 			if( natt )
-			{
-				log.txt( LLOG_INFO, 
-					"ii : phase1 id match ( natt prevents ip match )\n" );
-			}
+				expl = expl_natt;
 			else
-			{
 				if( ids.addr.s_addr != idt.addr.s_addr )
 					match = false;
-			}
 
 			break;
 		}
@@ -291,10 +311,7 @@ bool _IKED::cmp_ph1id( IKE_PH1ID & idt, IKE_PH1ID & ids, bool natt )
 				}
 			}
 			else
-			{
-				log.txt( LLOG_INFO, 
-					"ii : phase1 id match ( cert check only )\n" );
-			}
+				expl = expl_cert;
 
 			break;
 		}
@@ -304,9 +321,7 @@ bool _IKED::cmp_ph1id( IKE_PH1ID & idt, IKE_PH1ID & ids, bool natt )
 			log.txt( LLOG_ERROR, 
 				"!! : phase1 id mismatch ( internal error )\n" );
 
-			match = false;
-
-			break;
+			return false;
 		}
 	}
 
@@ -319,8 +334,10 @@ bool _IKED::cmp_ph1id( IKE_PH1ID & idt, IKE_PH1ID & ids, bool natt )
 		char txtid[ LIBIKE_MAX_TEXTP1ID ];
 		text_ph1id( txtid, &ids );
 
-		log.txt( LLOG_INFO, 
-			"ii : phase1 id match ( %s )\n",
+		log.txt( LLOG_INFO,
+			"ii : phase1 id match %s\n"
+			"ii : received = %s\n",
+			expl,
 			txtid );
 	}
 	else
@@ -337,9 +354,10 @@ bool _IKED::cmp_ph1id( IKE_PH1ID & idt, IKE_PH1ID & ids, bool natt )
 
 
 		log.txt( LLOG_ERROR, 
-			"!! : phase1 id mismatch ( src != trg )\n"
-			"!! : src = %s\n"
-			"!! : trg = %s\n",
+			"!! : phase1 id mismatch %s\n"
+			"!! : received = %s\n"
+			"!! : expected = %s\n",
+			expl,
 			txtid_s,
 			txtid_t );
 	}
