@@ -87,7 +87,7 @@ long _IKED::phase1_gen_prop( IDB_PH1 * ph1 )
 	//
 
 	IKE_PROPOSAL * peerprop;
-	if( !ph1->tunnel->peer->prop_list.get( &peerprop, 0, ISAKMP_PROTO_ISAKMP ) )
+	if( !ph1->tunnel->peer->proposals.get( &peerprop, 0, ISAKMP_PROTO_ISAKMP ) )
 		return LIBIKE_FAILED;
 
 	//
@@ -304,7 +304,7 @@ long _IKED::phase1_sel_prop( IDB_PH1 * ph1 )
 
 								if( ltemp.life_sec != rtemp.life_sec )
 								{
-									log.txt( LOG_INFO,
+									log.txt( LLOG_INFO,
 										"ii : adjusting %s lifetime %i -> %i ( obey )\n",
 										find_name( NAME_PROTOCOL, ltemp.proto ),
 										ltemp.life_sec,
@@ -322,7 +322,7 @@ long _IKED::phase1_sel_prop( IDB_PH1 * ph1 )
 
 								if( ltemp.life_sec > rtemp.life_sec )
 								{
-									log.txt( LOG_INFO,
+									log.txt( LLOG_INFO,
 										"ii : adjusting %s lifetime %i -> %i ( claim )\n",
 										find_name( NAME_PROTOCOL, ltemp.proto ),
 										ltemp.life_sec,
@@ -337,7 +337,7 @@ long _IKED::phase1_sel_prop( IDB_PH1 * ph1 )
 
 								if( ltemp.life_sec < rtemp.life_sec )
 								{
-									log.txt( LOG_INFO,
+									log.txt( LLOG_INFO,
 										"ii : using responder %s lifetime %i seconds, initiators is longer ( claim )\n",
 										find_name( NAME_PROTOCOL, ltemp.proto ),
 										ltemp.life_sec );
@@ -352,7 +352,7 @@ long _IKED::phase1_sel_prop( IDB_PH1 * ph1 )
 
 								if( ltemp.life_sec > rtemp.life_sec )
 								{
-									log.txt( LOG_INFO,
+									log.txt( LLOG_INFO,
 										"ii : adjusting %s lifetime %i -> %i ( strict )\n",
 										find_name( NAME_PROTOCOL, ltemp.proto ),
 										ltemp.life_sec,
@@ -396,7 +396,7 @@ bool _IKED::phase1_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( proposal1->proto != proposal2->proto )
 	{
-		log.txt( LOG_DEBUG,
+		log.txt( LLOG_DEBUG,
 			"ii : unmatched phase1 proposal/transform\n"
 			"ii : protocol ( %s != %s )\n",
 			find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -420,7 +420,7 @@ bool _IKED::phase1_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 		default:
 		{
-			log.txt( LOG_DEBUG,
+			log.txt( LLOG_DEBUG,
 				"ii : internal error, phase1 protocol unknown %i\n",
 				proposal2->proto );
 
@@ -434,7 +434,7 @@ bool _IKED::phase1_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( proposal1->xform != proposal2->xform )
 	{
-		log.txt( LOG_DEBUG,
+		log.txt( LLOG_DEBUG,
 			"ii : unmatched %s proposal/transform\n"
 			"ii : crypto transform type ( %s != %s )\n",
 			find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -450,7 +450,7 @@ bool _IKED::phase1_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( proposal1->ciph_id != proposal2->ciph_id )
 	{
-		log.txt( LOG_DEBUG,
+		log.txt( LLOG_DEBUG,
 			"ii : unmatched %s proposal/transform\n"
 			"ii : cipher type ( %s != %s )\n",
 			find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -466,7 +466,7 @@ bool _IKED::phase1_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( proposal1->ciph_kl != proposal2->ciph_kl )
 	{
-		log.txt( LOG_DEBUG,
+		log.txt( LLOG_DEBUG,
 			"ii : unmatched %s proposal/transform\n"
 			"ii : key length ( %i != %i )\n",
 			find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -482,7 +482,7 @@ bool _IKED::phase1_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( proposal1->hash_id != proposal2->hash_id )
 	{
-		log.txt( LOG_DEBUG,
+		log.txt( LLOG_DEBUG,
 			"ii : unmatched %s proposal/transform\n"
 			"ii : hash type ( %s != %s )\n",
 			find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -498,7 +498,7 @@ bool _IKED::phase1_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( proposal1->dhgr_id != proposal2->dhgr_id )
 	{
-		log.txt( LOG_DEBUG,
+		log.txt( LLOG_DEBUG,
 			"ii : unmatched %s proposal/transform\n"
 			"ii : dh group description ( %s != %s )\n",
 			find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -514,7 +514,7 @@ bool _IKED::phase1_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( proposal1->auth_id != proposal2->auth_id )
 	{
-		log.txt( LOG_DEBUG,
+		log.txt( LLOG_DEBUG,
 			"ii : unmatched %s proposal/transform\n"
 			"ii : hmac type ( %s != %s )\n",
 			find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -540,7 +540,7 @@ bool _IKED::phase1_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 			{
 				if( proposal1->life_sec < proposal2->life_sec )
 				{
-					log.txt( LOG_DEBUG,
+					log.txt( LLOG_DEBUG,
 						"ii : unmatched %s proposal/transform\n"
 						"ii : lifetime seconds ( %i < %i strict )\n",
 						find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -557,7 +557,7 @@ bool _IKED::phase1_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 			{
 				if( proposal1->life_sec != proposal2->life_sec )
 				{
-					log.txt( LOG_DEBUG,
+					log.txt( LLOG_DEBUG,
 						"ii : unmatched %s proposal/transform\n"
 						"ii : lifetime seconds ( %i != %i exact )\n",
 						find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -578,7 +578,7 @@ bool _IKED::phase1_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( initiator )
 		if( proposal1->tnumb != proposal2->tnumb )
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : peer violates RFC, transform number mismatch ( %i != %i )\n",
 				proposal1->tnumb,
 				proposal2->tnumb );
@@ -593,7 +593,7 @@ bool _IKED::phase1_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 	else
 		sprintf_s( klentxt, 32, "default" );
 
-	log.txt( LOG_INFO,
+	log.txt( LLOG_INFO,
 		"ii : matched %s proposal #%i transform #%i\n"
 		"ii : - transform    = %s\n"
 		"ii : - cipher type  = %s\n"
@@ -658,7 +658,7 @@ long _IKED::phase2_gen_prop( IDB_PH2 * ph2, IDB_POLICY * policy )
 				//
 
 				IKE_PROPOSAL * peerprop;
-				if( !ph2->tunnel->peer->prop_list.get( &peerprop, 0, ISAKMP_PROTO_IPSEC_AH ) )
+				if( !ph2->tunnel->peer->proposals.get( &peerprop, 0, ISAKMP_PROTO_IPSEC_AH ) )
 					break;
 
 				//
@@ -752,7 +752,7 @@ long _IKED::phase2_gen_prop( IDB_PH2 * ph2, IDB_POLICY * policy )
 				//
 
 				IKE_PROPOSAL * peerprop;
-				if( !ph2->tunnel->peer->prop_list.get( &peerprop, 0, ISAKMP_PROTO_IPSEC_ESP ) )
+				if( !ph2->tunnel->peer->proposals.get( &peerprop, 0, ISAKMP_PROTO_IPSEC_ESP ) )
 					break;
 
 				//
@@ -849,14 +849,17 @@ long _IKED::phase2_gen_prop( IDB_PH2 * ph2, IDB_POLICY * policy )
 
 							if( policy->xforms[ xindex ].mode == IPSEC_MODE_TUNNEL )
 							{
-								switch( ph2->tunnel->natt_v )
+								switch( ph2->tunnel->natt_version )
 								{
 									case IPSEC_NATT_NONE:
 										proposal.encap = ISAKMP_ENCAP_TUNNEL;
 										break;
 
+									case IPSEC_NATT_V00:
+									case IPSEC_NATT_V01:
 									case IPSEC_NATT_V02:
-										proposal.encap = ISAKMP_ENCAP_V02_UDP_TUNNEL;
+									case IPSEC_NATT_V03:
+										proposal.encap = ISAKMP_ENCAP_VXX_UDP_TUNNEL;
 										break;
 
 									case IPSEC_NATT_RFC:
@@ -866,14 +869,17 @@ long _IKED::phase2_gen_prop( IDB_PH2 * ph2, IDB_POLICY * policy )
 							}
 							else
 							{
-								switch( ph2->tunnel->natt_v )
+								switch( ph2->tunnel->natt_version )
 								{
 									case IPSEC_NATT_NONE:
 										proposal.encap = ISAKMP_ENCAP_TRANSPORT;
 										break;
 
+									case IPSEC_NATT_V00:
+									case IPSEC_NATT_V01:
 									case IPSEC_NATT_V02:
-										proposal.encap = ISAKMP_ENCAP_V02_UDP_TRANSPORT;
+									case IPSEC_NATT_V03:
+										proposal.encap = ISAKMP_ENCAP_VXX_UDP_TRANSPORT;
 										break;
 
 									case IPSEC_NATT_RFC:
@@ -936,7 +942,7 @@ long _IKED::phase2_gen_prop( IDB_PH2 * ph2, IDB_POLICY * policy )
 				//
 
 				IKE_PROPOSAL * peerprop;
-				if( !ph2->tunnel->peer->prop_list.get( &peerprop, 0, ISAKMP_PROTO_IPCOMP ) )
+				if( !ph2->tunnel->peer->proposals.get( &peerprop, 0, ISAKMP_PROTO_IPCOMP ) )
 					break;
 
 				//
@@ -1003,8 +1009,8 @@ long _IKED::phase2_sel_prop( IDB_PH2 * ph2 )
 	// bundle to a local proposal bundle
 	//
 
-	IKE_PLIST plist_l;
-	IKE_PLIST plist_r;
+	IDB_LIST_PROPOSAL plist_l;
+	IDB_LIST_PROPOSAL plist_r;
 
 	//
 	// step through our local bundles
@@ -1155,7 +1161,7 @@ long _IKED::phase2_sel_prop( IDB_PH2 * ph2 )
 
 							if( lproposal->life_sec != rproposal->life_sec )
 							{
-								log.txt( LOG_INFO,
+								log.txt( LLOG_INFO,
 									"ii : adjusting %s lifetime %i -> %i ( obey )\n",
 									find_name( NAME_PROTOCOL, lproposal->proto ),
 									lproposal->life_sec,
@@ -1173,7 +1179,7 @@ long _IKED::phase2_sel_prop( IDB_PH2 * ph2 )
 
 							if( lproposal->life_sec > rproposal->life_sec )
 							{
-								log.txt( LOG_INFO,
+								log.txt( LLOG_INFO,
 									"ii : adjusting %s lifetime %i -> %i ( claim )\n",
 									find_name( NAME_PROTOCOL, lproposal->proto ),
 									lproposal->life_sec,
@@ -1188,7 +1194,7 @@ long _IKED::phase2_sel_prop( IDB_PH2 * ph2 )
 
 							if( lproposal->life_sec < rproposal->life_sec )
 							{
-								log.txt( LOG_INFO,
+								log.txt( LLOG_INFO,
 									"ii : using responder %s lifetime %i seconds, initiators is longer ( claim )\n",
 									find_name( NAME_PROTOCOL, lproposal->proto ),
 									lproposal->life_sec );
@@ -1205,7 +1211,7 @@ long _IKED::phase2_sel_prop( IDB_PH2 * ph2 )
 
 							if( lproposal->life_sec > rproposal->life_sec )
 							{
-								log.txt( LOG_INFO,
+								log.txt( LLOG_INFO,
 									"ii : adjusting %s lifetime %i -> %i ( strict )\n",
 									find_name( NAME_PROTOCOL, lproposal->proto ),
 									lproposal->life_sec,
@@ -1240,7 +1246,7 @@ bool _IKED::phase2_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( proposal1->proto != proposal2->proto )
 	{
-		log.txt( LOG_DEBUG,
+		log.txt( LLOG_DEBUG,
 			"ii : unmatched proposal protocol\n"
 			"ii : protocol ( %s != %s )\n",
 			find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -1272,7 +1278,7 @@ bool _IKED::phase2_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 		default:
 		{
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : internal error, phase2 protocol unknown %i\n",
 				proposal2->proto );
 
@@ -1286,7 +1292,7 @@ bool _IKED::phase2_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( proposal1->xform != proposal2->xform )
 	{
-		log.txt( LOG_DEBUG,
+		log.txt( LLOG_DEBUG,
 			"ii : unmatched %s proposal/transform\n"
 			"ii : crypto transform type ( %s != %s )\n",
 			find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -1302,7 +1308,7 @@ bool _IKED::phase2_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( proposal1->ciph_kl != proposal2->ciph_kl )
 	{
-		log.txt( LOG_DEBUG,
+		log.txt( LLOG_DEBUG,
 			"ii : unmatched %s proposal/transform\n"
 			"ii : key length ( %i != %i )\n",
 			find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -1318,7 +1324,7 @@ bool _IKED::phase2_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( proposal1->encap != proposal2->encap )
 	{
-		log.txt( LOG_DEBUG,
+		log.txt( LLOG_DEBUG,
 			"ii : unmatched %s proposal/transform\n"
 			"ii : encapsulation mode ( %s != %s )\n",
 			find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -1334,7 +1340,7 @@ bool _IKED::phase2_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( proposal1->hash_id != proposal2->hash_id )
 	{
-		log.txt( LOG_DEBUG,
+		log.txt( LLOG_DEBUG,
 			"ii : unmatched %s proposal/transform\n"
 			"ii : msg auth ( %s != %s )\n",
 			find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -1350,7 +1356,7 @@ bool _IKED::phase2_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 
 	if( proposal1->dhgr_id != proposal2->dhgr_id )
 	{
-		log.txt( LOG_DEBUG,
+		log.txt( LLOG_DEBUG,
 			"ii : unmatched %s proposal/transform\n"
 			"ii : pfs dh group description ( %s != %s )\n",
 			find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -1376,7 +1382,7 @@ bool _IKED::phase2_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 			{
 				if( proposal1->life_sec < proposal2->life_sec )
 				{
-					log.txt( LOG_DEBUG,
+					log.txt( LLOG_DEBUG,
 						"ii : unmatched %s proposal/transform\n"
 						"ii : lifetime seconds ( %i < %i strict )\n",
 						find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -1393,7 +1399,7 @@ bool _IKED::phase2_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 			{
 				if( proposal1->life_sec != proposal2->life_sec )
 				{
-					log.txt( LOG_DEBUG,
+					log.txt( LLOG_DEBUG,
 						"ii : unmatched %s proposal/transform\n"
 						"ii : lifetime seconds ( %i != %i exact )\n",
 						find_name( NAME_PROTOCOL, proposal1->proto ),
@@ -1415,13 +1421,13 @@ bool _IKED::phase2_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 	if( initiator )
 	{
 		if( proposal1->pnumb != proposal2->pnumb )
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : peer violates RFC, proposal number mismatch ( %i != %i )\n",
 				proposal1->pnumb,
 				proposal2->pnumb );
 
 		if( proposal1->tnumb != proposal2->tnumb )
-			log.txt( LOG_ERROR,
+			log.txt( LLOG_ERROR,
 				"!! : peer violates RFC, transform number mismatch ( %i != %i )\n",
 				proposal1->tnumb,
 				proposal2->tnumb );
@@ -1437,7 +1443,7 @@ bool _IKED::phase2_cmp_prop( IKE_PROPOSAL * proposal1, IKE_PROPOSAL * proposal2,
 	else
 		sprintf_s( klentxt, 32, "default" );
 
-	log.txt( LOG_INFO,
+	log.txt( LLOG_INFO,
 		"ii : matched %s proposal #%i transform #%i\n"
 		"ii : - transform    = %s\n"
 		"ii : - key length   = %s\n"
