@@ -185,7 +185,7 @@ YY_DECL;
 
 #define VEND_SSOFT		{ 0xf1, 0x4b, 0x94, 0xb7, 0xbf, 0xf1, 0xfe, 0xf0, 0x27, 0x73, 0xb8, 0xc4, 0x9f, 0xed, 0xed, 0x26 }
 #define VEND_KAME		{ 0x70, 0x03, 0xcb, 0xc1, 0x09, 0x7d, 0xbe, 0x9c, 0x26, 0x00, 0xba, 0x69, 0x83, 0xbc, 0x8b, 0x35 }
-#define	VEND_UNITY		{ 0x12, 0xf5, 0xf2, 0x8c, 0x45, 0x71, 0x68, 0xa9, 0x70, 0x2d, 0x9f, 0xe2, 0x74, 0xcc, 0x01, 0x00 }
+#define	VEND_UNITY		{ 0x12, 0xf5, 0xf2, 0x8c, 0x45, 0x71, 0x68, 0xa9, 0x70, 0x2d, 0x9f, 0xe2, 0x74, 0xcc }
 #define VEND_NETSC		{ 0x16, 0x6f, 0x93, 0x2d, 0x55, 0xeb, 0x64, 0xd8, 0xe4, 0xdf, 0x4f, 0xd3, 0x7e, 0x23, 0x13, 0xf0, 0xd0, 0xfd, 0x84, 0x51 }
 #define VEND_ZWALL		{ 0x62, 0x50, 0x27, 0x74, 0x9d, 0x5a, 0xb9, 0x7f, 0x56, 0x16, 0xc1, 0x60, 0x27, 0x65, 0xcf, 0x48, 0x0a, 0x3b, 0x7d, 0x0b }
 #define VEND_SWIND		{ 0x84, 0x04, 0xad, 0xf9, 0xcd, 0xa0, 0x57, 0x60, 0xb2, 0xca, 0x29, 0x2e, 0x4b, 0xff, 0x53, 0x7b }
@@ -252,37 +252,36 @@ YY_DECL;
 #define XSTATE_RECV_SI			0x000001000
 #define XSTATE_RECV_CT			0x000002000
 #define XSTATE_RECV_CR			0x000004000
-#define XSTATE_RECV_NDL			0x000008000
-#define XSTATE_RECV_NDR			0x000010000
-#define XSTATE_RECV_IDL			0x000020000
-#define XSTATE_RECV_IDR			0x000040000
-#define XSTATE_RECV_HA			0x000080000
-#define XSTATE_RECV_LP			0x000100000
-#define XSTATE_SENT_LP			0x000200000
+#define XSTATE_RECV_ND			0x000008000
+#define XSTATE_RECV_IDL			0x000010000
+#define XSTATE_RECV_IDR			0x000020000
+#define XSTATE_RECV_HA			0x000040000
+#define XSTATE_RECV_LP			0x000080000
+#define XSTATE_SENT_LP			0x000100000
 
-#define LSTATE_HASSPI			0x000000001		// pfkey spi obtained
-#define LSTATE_CHKPROP			0x000000002		// proposal verified
-#define LSTATE_CHKHASH			0x000000004		// hash verified
-#define LSTATE_CHKIDS			0x000000008		// identity verified
-#define LSTATE_GENNATD			0x000000010		// natt discovery generated
-#define LSTATE_HASKEYS			0x000000020		// keys generated
-#define LSTATE_CLAIMLT			0x000000040		// claim reponder lifetime
+#define CSTATE_RECV_XUSER		0x000000001
+#define CSTATE_SENT_XUSER		0x000000002
+#define CSTATE_RECV_XPASS		0x000000004
+#define CSTATE_SENT_XPASS		0x000000008
+#define CSTATE_RECV_XRSLT		0x000000010
+#define CSTATE_SENT_XRSLT		0x000000020
+#define CSTATE_RECV_XCONF		0x000000040
+#define CSTATE_SENT_XCONF		0x000000080
+#define CSTATE_RECV_ACK			0x000000100
+#define CSTATE_SENT_ACK			0x000000200
+
+#define LSTATE_CHKPROP			0x000000001		// proposal verified
+#define LSTATE_CHKHASH			0x000000002		// hash verified
+#define LSTATE_CHKIDS			0x000000004		// identity verified
+#define LSTATE_GENNATD			0x000000008		// natt discovery generated
+#define LSTATE_HASKEYS			0x000000010		// keys generated
+#define LSTATE_CLAIMLT			0x000000020		// claim reponder lifetime
 
 #define TSTATE_NATT_FLOAT		0x000000001
 #define TSTATE_INITIALIZED		0x000000002
-#define TSTATE_RECV_XUSER		0x000000004
-#define TSTATE_SENT_XUSER		0x000000008
-#define TSTATE_RECV_XPASS		0x000000010
-#define TSTATE_SENT_XPASS		0x000000020
-#define TSTATE_RECV_XAUTH		( TSTATE_RECV_XUSER | TSTATE_RECV_XPASS )
-#define TSTATE_SENT_XAUTH		( TSTATE_SENT_XUSER | TSTATE_SENT_XPASS )
-#define TSTATE_RECV_XRSLT		0x000000040
-#define TSTATE_SENT_XRSLT		0x000000080
-#define TSTATE_RECV_CONFIG		0x000000100
-#define TSTATE_SENT_CONFIG		0x000000200
-#define TSTATE_RECV_ACK			0x000000400
-#define TSTATE_SENT_ACK			0x000000800
-#define TSTATE_VNET_ENABLE		0x000001000
+#define TSTATE_VNET_CONFIG		0x000000004
+#define TSTATE_VNET_ENABLE		0x000000008
+#define TSTATE_POLICY_INIT		0x000000010
 
 #define DSTATE_ACTIVE			0
 #define DSTATE_TERMINATE		1
@@ -642,6 +641,7 @@ typedef class _IKED
 	long	phase1_chk_hash( IDB_PH1 * ph1 );
 	long	phase1_chk_sign( IDB_PH1 * ph1 );
 	long	phase1_gen_natd( IDB_PH1 * ph1 );
+	bool	phase1_add_natd( IDB_PH1 * ph1, PACKET_IKE & packet, uint8_t next );
 	bool	phase1_chk_natd( IDB_PH1 * ph1 );
 	long	phase1_chk_idr( IDB_PH1 * ph1 );
 
@@ -656,10 +656,23 @@ typedef class _IKED
 	long	phase2_chk_params( IDB_PH1 * ph1, IDB_PH2 * ph2, PACKET_IKE & packet );
 	long	phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2 );
 	long	phase2_gen_keys( IDB_PH1 * ph1, IDB_PH2 * ph2, long dir, IKE_PROPOSAL * proposal, BDATA & shared );
-	long	phase2_gen_iv( IDB_PH1 * ph1, unsigned long msgid, BDATA & iv );
 
 	// config exchange helper functions
 
+	bool	config_client_xauth_recv( IDB_CFG * cfg, IDB_PH1 * ph1 );
+	bool	config_client_xauth_send( IDB_CFG * cfg, IDB_PH1 * ph1 );
+	bool	config_client_xconf_pull_recv( IDB_CFG * cfg, IDB_PH1 * ph1 );
+	bool	config_client_xconf_pull_send( IDB_CFG * cfg, IDB_PH1 * ph1 );
+	bool	config_client_xconf_push_recv( IDB_CFG * cfg, IDB_PH1 * ph1 );
+	bool	config_client_xconf_push_send( IDB_CFG * cfg, IDB_PH1 * ph1 );
+
+	bool	config_server_xauth_recv( IDB_CFG * cfg, IDB_PH1 * ph1 );
+	bool	config_server_xauth_send( IDB_CFG * cfg, IDB_PH1 * ph1 );
+	bool	config_server_xconf_pull_recv( IDB_CFG * cfg, IDB_PH1 * ph1 );
+	bool	config_server_xconf_pull_send( IDB_CFG * cfg, IDB_PH1 * ph1 );
+	bool	config_server_xconf_push_recv( IDB_CFG * cfg, IDB_PH1 * ph1 );
+	bool	config_server_xconf_push_send( IDB_CFG * cfg, IDB_PH1 * ph1 );
+
 	long	config_xconf_set( IDB_CFG * cfg, long & setmask, long nullmask, VENDOPTS vendopts );
 	long	config_xconf_get( IDB_CFG * cfg, long & getmask, long readmask, VENDOPTS vendopts );
 
@@ -766,8 +779,8 @@ typedef class _IKED
 	// execution thread loops
 	//
 
-	void	loop_ref_inc( char * name );
-	void	loop_ref_dec( char * name );
+	void	loop_ref_inc( const char * name );
+	void	loop_ref_dec( const char * name );
 
 	long	loop_ipc_server();
 	long	loop_ipc_client( IKEI * ikei );
