@@ -412,7 +412,7 @@ typedef class _IDB_PEER : public IDB_RC_ENTRY, public IKE_PEER
 	IDB_LIST_PROPOSAL	proposals;
 	IDB_LIST_NETMAP		netmaps;
 
-	virtual	char *			name();
+	virtual	const char *	name();
 	virtual IDB_RC_LIST *	list();
 
 	_IDB_PEER( IKE_PEER * set_peer );
@@ -478,7 +478,7 @@ typedef class _IDB_TUNNEL : public IDB_RC_ENTRY
 	ITH_EVENT_TUNNATT	event_natt;
 	ITH_EVENT_TUNSTATS	event_stats;
 
-	virtual	char *			name();
+	virtual	const char *	name();
 	virtual IDB_RC_LIST *	list();
 
 	virtual void	beg();
@@ -508,10 +508,12 @@ typedef class _IDB_POLICY : public IDB_RC_ENTRY, public PFKI_SPINFO
 {
 	public:
 
-	bool		route;
-	bool		nailed;
+	bool			route_added;
+	IPROUTE_ENTRY	route_entry;
+	bool			nailed;
+	bool			initial;
 
-	virtual	char *			name();
+	virtual	const char *	name();
 	virtual IDB_RC_LIST *	list();
 
 	virtual void	beg();
@@ -566,28 +568,12 @@ typedef class _IDB_XCH : public IDB_RC_ENTRY
 	long		lstate;
 	long		xstate;
 
-	DH *		dh;
-	long		dh_size;
-
-	BDATA		nonce_l;
-	BDATA		nonce_r;
-
-	BDATA		xl;
-	BDATA		xr;
-
-	long		hash_size;
 	BDATA		hash_l;
 	BDATA		hash_r;
 
 	BDATA		hda;		// hash data accumulator
 	BDATA		iv;
 
-	//
-	// FIXME : only use lists in classes that need them
-	//
-
-	IDB_LIST_PROPOSAL	plist_l;
-	IDB_LIST_PROPOSAL	plist_r;
 	IDB_LIST_NOTIFY		notifications;
 
 	ITH_EVENT_RESEND	event_resend;
@@ -598,17 +584,47 @@ typedef class _IDB_XCH : public IDB_RC_ENTRY
 	XCH_STATUS	status();
 	XCH_STATUS	status( XCH_STATUS status, XCH_ERRORCODE errorcode, uint16_t notifycode );
 
+	void	new_msgid();
+	bool	new_msgiv( IDB_PH1 * ph1 );
+
+	bool	resend();
 	bool	resend_queue( PACKET_IP & packet );
+	void	resend_purge();
 	bool	resend_sched( bool lock );
-	void	resend_clear( bool lock ) ;
+	void	resend_clear( bool lock, bool purge );
 
 }IDB_XCH;
 
+//==============================================================================
+// ike generic sa exchange handle class
+//
+
+typedef class _IDB_XCH_SA : public IDB_XCH
+{
+	public:
+
+	DH *		dh;
+	long		dh_size;
+
+	BDATA		xl;
+	BDATA		xr;
+
+	BDATA		nonce_l;
+	BDATA		nonce_r;
+
+	IDB_LIST_PROPOSAL	plist_l;
+	IDB_LIST_PROPOSAL	plist_r;
+
+	_IDB_XCH_SA();
+	virtual ~_IDB_XCH_SA();
+
+}IDB_XCH_SA;
+
 //==============================================================================
 // ike phase1 exchange handle class
 //
 
-typedef class _IDB_PH1 : public IDB_XCH
+typedef class _IDB_PH1 : public IDB_XCH_SA
 {
 	virtual void	beg();
 	virtual void	end();
@@ -617,6 +633,7 @@ typedef class _IDB_PH1 : public IDB_XCH
 
 	const EVP_CIPHER *	evp_cipher;
 	const EVP_MD *		evp_hash;
+	long				hash_size;
 
 	IKE_COOKIES	cookies;
 
@@ -629,9 +646,6 @@ typedef class _IDB_PH1 : public IDB_XCH
 	long	natt_version;	// version negotiated
 	uint8_t	natt_pldtype;	// payload identifier
 
-	bool	natted_l;	// local address is natted
-	bool	natted_r;	// remote address is natted
-
 	uint8_t		ctype_l;	// local certificate type
 	uint8_t		ctype_r;	// remote certificate type
 
@@ -644,10 +658,8 @@ typedef class _IDB_PH1 : public IDB_XCH
 	BDATA	idi;
 	BDATA	idr;
 
-	BDATA	natd_ls;
-	BDATA	natd_ld;
-	BDATA	natd_rs;
-	BDATA	natd_rd;
+	IDB_LIST_BDATA	natd_hash_l;
+	IDB_LIST_BDATA	natd_hash_r;
 
 	IDB_LIST_CERT	certs_r;
 	BDATA			sign_r;
@@ -663,7 +675,7 @@ typedef class _IDB_PH1 : public IDB_XCH
 
 	// sub class functions
 
-	virtual	char *			name();
+	virtual	const char *	name();
 	virtual IDB_RC_LIST *	list();
 
 	// class functions
@@ -705,7 +717,7 @@ typedef class _IDB_LIST_PH1 : public IDB_LIST_IKED
 // ike phase2 exchange handle class
 //
 
-typedef class _IDB_PH2 : public IDB_XCH
+typedef class _IDB_PH2 : public IDB_XCH_SA
 {
 	public:
 
@@ -727,7 +739,7 @@ typedef class _IDB_PH2 : public IDB_XCH
 
 	// sub class functions
 
-	virtual	char *			name();
+	virtual	const char *	name();
 	virtual IDB_RC_LIST *	list();
 
 	virtual void	beg();
@@ -778,9 +790,14 @@ typedef class _IDB_CFG : public IDB_XCH
 
 	public:
 
+	// this should never be accessed
+	// directly, only for comparison
+
+	IDB_PH1 *	ph1ref;
+
 	// sub class functions
 
-	virtual	char *			name();
+	virtual	const char *	name();
 	virtual IDB_RC_LIST *	list();
 
 	virtual void	beg();
@@ -788,7 +805,7 @@ typedef class _IDB_CFG : public IDB_XCH
 
 	// class functions
 
-	_IDB_CFG( IDB_TUNNEL * tunnel, bool set_initiator, unsigned long set_msgid );
+	_IDB_CFG( IDB_PH1 * set_ph1ref, bool set_initiator );
 	virtual ~_IDB_CFG();
 
 	BDATA		hash;
@@ -822,8 +839,7 @@ typedef class _IDB_LIST_CFG : public IDB_LIST_IKED
 	bool find(
 			bool lock,
 			IDB_CFG ** cfg,
-			IDB_TUNNEL * tunnel,
-			unsigned long msgid );
+			IDB_PH1 * ph1 );
 
 }IDB_LIST_CFG;
 
@@ -837,7 +853,7 @@ typedef class _IDB_INF : public IDB_XCH
 
 	// sub class functions
 
-	virtual	char *			name();
+	virtual	const char *	name();
 	virtual IDB_RC_LIST *	list();
 
 	virtual void	beg();
