@@ -121,6 +121,8 @@ typedef struct _IKEI_BASIC
 
 typedef struct _IKEI_STATS
 {
+	IKE_SADDR	peer;
+
 	long	sa_good;
 	long	sa_fail;
 	long	sa_dead;
@@ -152,7 +154,7 @@ typedef class DLX _IKEI_MSG : public BDATA
 
 	long	get_status( long * status, BDATA * str );
 	long	set_status( long status, BDATA * str );
-	long	set_status( long status, char * str );
+	long	set_status( long status, const char * str );
 
 	long	get_stats( IKEI_STATS * stats );
 	long	set_stats( IKEI_STATS * stats );
