@@ -45,7 +45,7 @@
 #ifdef WIN32
 # define MPR50 1
 # include <winsock2.h>
-# include <windows.h>
+# include <ws2ipdef.h>
 # include <iphlpapi.h>
 # include <routprot.h>
 # include <inttypes.h>
@@ -491,6 +491,35 @@ typedef class DLX _IPQUEUE : private IDB_LIST
 
 }IPQUEUE;
 
+typedef struct _IPROUTE_ENTRY : public IDB_ENTRY
+{
+	bool	local;
+	in_addr	iface;
+	in_addr	addr;
+	in_addr	mask;
+	in_addr	next;
+	
+}IPROUTE_ENTRY;
+
+#ifndef WIN32
+
+typedef class DLX _IPROUTE_LIST : private IDB_LIST
+{
+	public:
+
+	_IPROUTE_LIST();
+	virtual ~_IPROUTE_LIST();
+
+	bool	add( IPROUTE_ENTRY & route );
+	bool	get( IPROUTE_ENTRY & route );
+
+	long	count();
+	void	clean();
+
+}IPROUTE_LIST;
+
+#endif
+
 typedef class DLX _IPROUTE
 {
 	private:
@@ -499,20 +528,27 @@ typedef class DLX _IPROUTE
 	unsigned long	osver_maj;
 	unsigned long	osver_min;
 
+#ifndef WIN32
+	IPROUTE_LIST	route_list;
+#endif
+
 	public:
 
-	bool	iface_2_addr( in_addr & iface, unsigned long & index );
-	bool	addr_2_iface( unsigned long & index, in_addr & iface );
+#ifdef WIN32
+	bool	iface_metric( unsigned long & metric, unsigned long index );
+	bool	iface_2_addr( in_addr & iface, unsigned long index );
+	bool	addr_2_iface( unsigned long & index, in_addr iface );
+#endif	
 
 	public:
 
 	_IPROUTE();
 
-	bool add( in_addr & iface, bool local, in_addr addr, in_addr mask, in_addr next );
-	bool del( in_addr & iface, bool local, in_addr addr, in_addr mask, in_addr next );
-	bool get( in_addr & iface, bool & local, in_addr & addr, in_addr & mask, in_addr & next );
+	bool add( IPROUTE_ENTRY & route );
+	bool del( IPROUTE_ENTRY & route );
+	bool get( IPROUTE_ENTRY & route );
 
-	bool best( in_addr & iface, bool & local, in_addr & addr, in_addr & mask, in_addr & next );
+	bool best( IPROUTE_ENTRY & route );
 
 	bool increment( in_addr addr, in_addr mask );
 	bool decrement( in_addr addr, in_addr mask );
