@@ -41,6 +41,98 @@
 
 #include "libip.h"
 
+void text_route( IPROUTE_ENTRY & route, char * text, bool dstonly = false )
+{
+	char txt_iface[ 24 ];
+	char txt_addr[ 24 ];
+	char txt_mask[ 24 ];
+	char txt_next[ 24 ];
+
+	if( dstonly )
+	{
+		strcpy( txt_addr, inet_ntoa( route.addr ));
+		strcpy( txt_mask, inet_ntoa( route.mask ));
+
+		sprintf( text, "%s/%s",
+			txt_addr, txt_mask );
+	}
+	else
+	{
+		strcpy( txt_iface, inet_ntoa( route.iface ));
+		strcpy( txt_addr, inet_ntoa( route.addr ));
+		strcpy( txt_mask, inet_ntoa( route.mask ));
+		strcpy( txt_next, inet_ntoa( route.next ));
+
+		sprintf( text, "%s/%s gw %s if %s",
+			txt_addr, txt_mask, txt_next, txt_iface );
+	}
+}
+
+//==============================================================================
+// Route list class
+//==============================================================================
+
+_IPROUTE_LIST::_IPROUTE_LIST()
+{
+}
+
+_IPROUTE_LIST::~_IPROUTE_LIST()
+{
+	clean();
+}
+
+bool _IPROUTE_LIST::add( IPROUTE_ENTRY & route )
+{
+	IPROUTE_ENTRY * tmp_route = new IPROUTE_ENTRY;
+	if( tmp_route == NULL )
+		return false;
+
+	*tmp_route = route;
+
+	add_entry( tmp_route );
+
+    return true;
+}
+
+bool _IPROUTE_LIST::get( IPROUTE_ENTRY & route )
+{
+	long index = 0;
+	for( ; index < count(); index++ )
+	{
+		IPROUTE_ENTRY * tmp_route = static_cast<IPROUTE_ENTRY*>( get_entry( index ) );
+		assert( tmp_route != NULL );
+
+		if( tmp_route->addr.s_addr != route.addr.s_addr )
+			continue;
+
+		if( tmp_route->mask.s_addr != route.mask.s_addr )
+			continue;
+
+		route = *tmp_route;
+
+		del_entry( tmp_route );
+		delete tmp_route;
+
+		return true;
+	}
+
+	return false;
+}
+
+long _IPROUTE_LIST::count()
+{
+    return IDB_LIST::count();
+}
+
+void _IPROUTE_LIST::clean()
+{
+    IDB_LIST::clean();
+}
+
+//==============================================================================
+// BSD specific route handling
+//==============================================================================
+
 #ifndef __linux__
 
 //
@@ -66,42 +158,84 @@ typedef struct _RTMSG
 }RTMSG;
 
 //
-// BSD route message result function
+// BSD route message send request function
 //
 
-bool rtmsg_result( RTMSG * rtmsg, in_addr * dst, in_addr * gwy, in_addr * msk, in_addr * ifa )
+int rtmsg_send( RTMSG * rtmsg )
 {
-	char *	cp = rtmsg->msg;
+	int s = socket( PF_ROUTE, SOCK_RAW, 0 );
+	if( s < 0 )
+		return -1;
+
+	long l = rtmsg->hdr.rtm_msglen += sizeof( rtmsg->hdr );
+
+	if( write( s, rtmsg, l ) < 0 )
+	{
+		close( s );
+		return -2;
+	}
+
+	return s;
+}
+
+//
+// BSD route message receive result function
+//
+
+bool rtmsg_recv( int s, int seq, IPROUTE_ENTRY * route )
+{
+	RTMSG rtmsg;
+	memset( &rtmsg, 0, sizeof( rtmsg ) );
+
+	// read route result message
+
+	pid_t pid = getpid();
+	long l;
+
+	do
+	{
+		l = read( s, &rtmsg, sizeof( rtmsg ) );
+		if( l < 0 )
+		{
+			close( s );
+			return false;
+		}
+	}
+	while( ( rtmsg.hdr.rtm_seq != seq ) || ( rtmsg.hdr.rtm_pid != pid ) );
+
+	close( s );
+
+	if( ( rtmsg.hdr.rtm_errno ) || ( rtmsg.hdr.rtm_msglen > l ) ||
+		( rtmsg.hdr.rtm_version != RTM_VERSION ) )
+		return false;
+
+	char *	cp = rtmsg.msg;
 
 	for( int i = 1; i; i <<= 1 )
 	{
-		if( i & rtmsg->hdr.rtm_addrs )
+		if( i & rtmsg.hdr.rtm_addrs )
 		{
 			struct sockaddr * sa = ( struct sockaddr * ) cp;
 
 			switch( i )
 			{
 				case RTA_DST:
-					if( dst != NULL )
-						*dst = ( ( sockaddr_in * ) cp )->sin_addr;
+					route->addr = ( ( sockaddr_in * ) cp )->sin_addr;
 					break;
 
 				case RTA_GATEWAY:
-					if( gwy != NULL )
-						*gwy = ( ( sockaddr_in * ) cp )->sin_addr;
+					route->next = ( ( sockaddr_in * ) cp )->sin_addr;
 					break;
 
 				case RTA_NETMASK:
-					if( msk != NULL )
-						*msk = ( ( sockaddr_in * ) cp )->sin_addr;
+					route->mask = ( ( sockaddr_in * ) cp )->sin_addr;
 					break;
 
 				case RTA_IFP:
 					break;
 
 				case RTA_IFA:
-					if( ifa != NULL )
-						*ifa = ( ( sockaddr_in * ) cp )->sin_addr;
+					route->iface = ( ( sockaddr_in * ) cp )->sin_addr;
 					break;
 			}
 
@@ -123,12 +257,8 @@ _IPROUTE::_IPROUTE()
 
 // add a route
 
-bool _IPROUTE::add( in_addr & iface, bool local, in_addr addr, in_addr mask, in_addr next )
+bool _IPROUTE::add( IPROUTE_ENTRY & route )
 {
-	int s = socket( PF_ROUTE, SOCK_RAW, 0 );
-	if( s == -1 )
-		return false;
-
 	// set route message header
 
 	RTMSG rtmsg;
@@ -140,10 +270,7 @@ bool _IPROUTE::add( in_addr & iface, bool local, in_addr addr, in_addr mask, in_
 	rtmsg.hdr.rtm_flags = RTF_UP | RTF_STATIC | RTF_GATEWAY;
 	rtmsg.hdr.rtm_addrs = RTA_DST | RTA_GATEWAY;
 
-//	if( mask.s_addr == 0xffffffff )
-//		rtmsg.hdr.rtm_flags |= RTF_HOST;
-//	else
-		rtmsg.hdr.rtm_addrs |= RTA_NETMASK;
+	rtmsg.hdr.rtm_addrs |= RTA_NETMASK;
 		
 	// add route destination
 
@@ -151,7 +278,7 @@ bool _IPROUTE::add( in_addr & iface, bool local, in_addr addr, in_addr mask, in_
 
 	dst->sin_family = AF_INET;
 	dst->sin_len = sizeof( sockaddr_in );
-	dst->sin_addr = addr;
+	dst->sin_addr = route.addr;
 
 	rtmsg.hdr.rtm_msglen += sizeof( sockaddr_in );
 
@@ -161,32 +288,25 @@ bool _IPROUTE::add( in_addr & iface, bool local, in_addr addr, in_addr mask, in_
 
 	gwy->sin_family = AF_INET;
 	gwy->sin_len = sizeof( sockaddr_in );
-	gwy->sin_addr = next;
+	gwy->sin_addr = route.next;
 
 	rtmsg.hdr.rtm_msglen += sizeof( sockaddr_in );
 
 	// add route netmask
 
-//	if( mask.s_addr != 0xffffffff )
-	{
-		sockaddr_in * msk = ( sockaddr_in * )( rtmsg.msg + rtmsg.hdr.rtm_msglen );
+	sockaddr_in * msk = ( sockaddr_in * )( rtmsg.msg + rtmsg.hdr.rtm_msglen );
 
-		msk->sin_family = AF_INET;
-		msk->sin_len = sizeof( sockaddr_in );
-		msk->sin_addr = mask;
+	msk->sin_family = AF_INET;
+	msk->sin_len = sizeof( sockaddr_in );
+	msk->sin_addr = route.mask;
 
-		rtmsg.hdr.rtm_msglen += sizeof( sockaddr_in );
-	}
+	rtmsg.hdr.rtm_msglen += sizeof( sockaddr_in );
 
 	// send route add message
 
-	long l = rtmsg.hdr.rtm_msglen += sizeof( rtmsg.hdr );
-
-	if( write( s, ( char * ) &rtmsg, l ) < 0 )
-	{
-		close( s );
+	int s = rtmsg_send( &rtmsg );
+	if( s < 0 )
 		return false;
-	}
 
 	close( s );
 
@@ -195,12 +315,8 @@ bool _IPROUTE::add( in_addr & iface, bool local, in_addr addr, in_addr mask, in_
 
 // delete a route 
 
-bool _IPROUTE::del( in_addr & iface, bool local, in_addr addr, in_addr mask, in_addr next )
+bool _IPROUTE::del( IPROUTE_ENTRY & route )
 {
-	int s = socket( PF_ROUTE, SOCK_RAW, 0 );
-	if( s == -1 )
-		return false;
-
 	// set route message header
 
 	RTMSG rtmsg;
@@ -212,10 +328,7 @@ bool _IPROUTE::del( in_addr & iface, bool local, in_addr addr, in_addr mask, in_
 	rtmsg.hdr.rtm_flags = RTF_UP | RTF_STATIC | RTF_GATEWAY;
 	rtmsg.hdr.rtm_addrs = RTA_DST | RTA_GATEWAY;
 
-//	if( mask.s_addr == 0xffffffff )
-//		rtmsg.hdr.rtm_flags |= RTF_HOST;
-//	else
-		rtmsg.hdr.rtm_addrs |= RTA_NETMASK;
+	rtmsg.hdr.rtm_addrs |= RTA_NETMASK;
 		
 	// add route destination
 
@@ -223,7 +336,7 @@ bool _IPROUTE::del( in_addr & iface, bool local, in_addr addr, in_addr mask, in_
 
 	dst->sin_family = AF_INET;
 	dst->sin_len = sizeof( sockaddr_in );
-	dst->sin_addr = addr;
+	dst->sin_addr = route.addr;
 
 	rtmsg.hdr.rtm_msglen += sizeof( sockaddr_in );
 
@@ -233,32 +346,25 @@ bool _IPROUTE::del( in_addr & iface, bool local, in_addr addr, in_addr mask, in_
 
 	gwy->sin_family = AF_INET;
 	gwy->sin_len = sizeof( sockaddr_in );
-	gwy->sin_addr = next;
+	gwy->sin_addr = route.next;
 
 	rtmsg.hdr.rtm_msglen += sizeof( sockaddr_in );
 
 	// add route netmask
 
-//	if( mask.s_addr != 0xffffffff )
-	{
-		sockaddr_in * msk = ( sockaddr_in * )( rtmsg.msg + rtmsg.hdr.rtm_msglen );
+	sockaddr_in * msk = ( sockaddr_in * )( rtmsg.msg + rtmsg.hdr.rtm_msglen );
 
-		msk->sin_family = AF_INET;
-		msk->sin_len = sizeof( sockaddr_in );
-		msk->sin_addr = mask;
+	msk->sin_family = AF_INET;
+	msk->sin_len = sizeof( sockaddr_in );
+	msk->sin_addr = route.mask;
 
-		rtmsg.hdr.rtm_msglen += sizeof( sockaddr_in );
-	}
+	rtmsg.hdr.rtm_msglen += sizeof( sockaddr_in );
 
 	// send route delete message
 
-	long l = rtmsg.hdr.rtm_msglen += sizeof( rtmsg.hdr );
-
-	if( write( s, ( char * ) &rtmsg, l ) < 0 )
-	{
-		close( s );
+	int s = rtmsg_send( &rtmsg );
+	if( s < 0 )
 		return false;
-	}
 
 	close( s );
 
@@ -267,19 +373,54 @@ bool _IPROUTE::del( in_addr & iface, bool local, in_addr addr, in_addr mask, in_
 
 // get a route ( by addr and mask )
 
-bool _IPROUTE::get( in_addr & iface, bool & local, in_addr & addr, in_addr & mask, in_addr & next )
+bool _IPROUTE::get( IPROUTE_ENTRY & route )
 {
-	return true;
+	// set route message header
+
+	RTMSG rtmsg;
+	memset( &rtmsg, 0, sizeof( rtmsg ) );
+
+	rtmsg.hdr.rtm_version = RTM_VERSION;
+	rtmsg.hdr.rtm_type = RTM_GET;
+	rtmsg.hdr.rtm_seq = ++seq;
+	rtmsg.hdr.rtm_flags = RTF_UP | RTF_STATIC | RTF_STATIC;
+	rtmsg.hdr.rtm_addrs = RTA_DST;
+
+	// add route destination
+
+	sockaddr_in * dst = ( sockaddr_in * ) rtmsg.msg;
+
+	dst->sin_family = AF_INET;
+	dst->sin_len = sizeof( sockaddr_in );
+	dst->sin_addr = route.addr;
+
+	rtmsg.hdr.rtm_msglen += sizeof( sockaddr_in );
+
+	// add route netmask
+
+	sockaddr_in * msk = ( sockaddr_in * )( rtmsg.msg + rtmsg.hdr.rtm_msglen );
+
+	msk->sin_family = AF_INET;
+	msk->sin_len = sizeof( sockaddr_in );
+	msk->sin_addr = route.mask;
+
+	rtmsg.hdr.rtm_msglen += sizeof( sockaddr_in );
+
+	// send route get message
+
+	int s = rtmsg_send( &rtmsg );
+	if( s < 0 )
+		return false;
+
+	// read route result message
+
+	return rtmsg_recv( s, seq, &route );
 }
 
 // get best route ( by address )
 
-bool _IPROUTE::best( in_addr & iface, bool & local, in_addr & addr, in_addr & mask, in_addr & next )
+bool _IPROUTE::best( IPROUTE_ENTRY & route )
 {
-	int s = socket( PF_ROUTE, SOCK_RAW, 0 );
-	if( s == -1 )
-		return false;
-
 	// set route message header
 
 	RTMSG rtmsg;
@@ -297,7 +438,7 @@ bool _IPROUTE::best( in_addr & iface, bool & local, in_addr & addr, in_addr & ma
 
 	dst->sin_family = AF_INET;
 	dst->sin_len = sizeof( sockaddr_in );
-	dst->sin_addr = addr;
+	dst->sin_addr = route.addr;
 
 	rtmsg.hdr.rtm_msglen += sizeof( sockaddr_in );
 
@@ -312,72 +453,101 @@ bool _IPROUTE::best( in_addr & iface, bool & local, in_addr & addr, in_addr & ma
 
 	// send route get message
 
-	long l = rtmsg.hdr.rtm_msglen += sizeof( rtmsg.hdr );
-
-	if( write( s, ( char * ) &rtmsg, l ) < 0 )
-	{
-		close( s );
+	int s = rtmsg_send( &rtmsg );
+	if( s < 0 )
 		return false;
-	}
-
-	int pid = getpid();
 
 	// read route result message
 
-	do
-	{
-		l = read( s, ( char * ) &rtmsg, sizeof( rtmsg ) );
-		if( l < 0 )
-		{
-			close( s );
-			return false;
-		}
-	}
-	while( ( rtmsg.hdr.rtm_seq != seq ) ||
-		   ( rtmsg.hdr.rtm_pid != pid ) );
+	return rtmsg_recv( s, seq, &route );
+}
 
-	close( s );
+//
+// BSD unix systems don't support multiple routes
+// to the same destination network. We go for the
+// lowest common denominator. Cache the existing
+// route information and delete the existing route
+// on increment. Retrieve and restore the previous
+// route on decrement. 
+//
 
-	if( ( rtmsg.hdr.rtm_errno ) ||
-		( rtmsg.hdr.rtm_msglen > l ) ||
-		( rtmsg.hdr.rtm_version != RTM_VERSION ) )
-		return false;
+// increment route costs
 
-	return rtmsg_result( &rtmsg, &addr, &next, &mask, &iface );
+bool _IPROUTE::increment( in_addr addr, in_addr mask )
+{
+	//
+	// locate the most specific route for the destination
+	//
+
+	IPROUTE_ENTRY route;
+	memset( &route, 0, sizeof( route ) );
+	route.addr = addr;
+	route.mask = mask;
+
+	if( !get( route ) )
+		return true;
+
+	//
+	// does this route match the destination exactly
+	//
+
+	if( route.addr.s_addr != addr.s_addr )
+		return true;
+
+	if( route.mask.s_addr != mask.s_addr )
+		return true;
+
+	//
+	// add a route entry to our route list
+	//
+
+	route_list.add( route );
+
+	//
+	// delete the existing route
+	//
+
+	return del( route );
 }
 
 // decrement route costs
 
-bool _IPROUTE::increment( in_addr addr, in_addr mask )
-{
-	return true;
-}
-
-// increment route costs
-
 bool _IPROUTE::decrement( in_addr addr, in_addr mask )
 {
-	return true;
-}
+	//
+	// locate the cached route info for the destination
+	//
 
-// flush arp table
+	IPROUTE_ENTRY route;
+	memset( &route, 0, sizeof( route ) );
+	route.addr = addr;
+	route.mask = mask;
 
-bool _IPROUTE::flusharp( in_addr & iface )
-{
-	return true;
+	if( !route_list.get( route ) )
+		return true;
+
+	//
+	// delete the restore the route for the destination
+	//
+
+	return add( route );
 }
 
 #else
 
+//==============================================================================
+// Linux specific route handling
+//==============================================================================
+
 //
-// Linux netlink message wrapper struct
+// netlink message wrapper struct
 //
 
 typedef struct _NLMSG
 {
-    struct nlmsghdr	hdr;
-    struct rtmsg	msg;
-    char		buff[ 1024 ];
+	struct nlmsghdr hdr;
+	struct rtmsg msg;
+	char buff[ 1024 ];
 
 }NLMSG;
 
@@ -417,7 +587,7 @@ unsigned int prefix_to_mask( int plen )
 	for( int i = 0; i < plen; i++ )
 	{
 		mask >>= 1;
-                mask |= 0x80000000;
+		mask |= 0x80000000;
 	}
 
 	return htonl( mask );
@@ -449,102 +619,127 @@ int rtmsg_send( NLMSG * nlmsg )
 	return s;
 }
 
-int rtmsg_recv( int s, in_addr * dst, in_addr * gwy, in_addr * msk, in_addr * ifa )
+bool rtmsg_recv( int s, IPROUTE_ENTRY & route )
 {
-	char	buf[ sizeof( NLMSG ) ];
-	memset( buf, 0, sizeof( buf ) );
+	char	buff[ sizeof( NLMSG ) ];
+	memset( &buff, 0, sizeof( NLMSG ) );
 
-	char *	p = buf;
-	int	nll = 0;
-	int	rtl = 0;
+	int rslt = recv( s, buff, sizeof( NLMSG ), 0 );
 
-	struct nlmsghdr * nlp;
-	struct rtmsg * rtp;
-	struct rtattr * rtap;
+	close( s );
 
-	int	rtn;
+	if( rslt <= 0 )
+		return false;
 
-	while( true )
+	struct nlmsghdr * nlmsg = ( struct nlmsghdr * ) buff;
+	int nllen = rslt;
+
+	while( NLMSG_OK( nlmsg, nllen ) )
 	{
-		rtn = recv( s, p, sizeof( buf ) - nll, 0 );
-		if( !rtn )
-			break;
+		// printf( "XX : netlink msg type = %i\n", nlmsg->nlmsg_type );
 
-		nlp = ( struct nlmsghdr * ) p;
-
-		if( nlp->nlmsg_type == NLMSG_ERROR )
-			return -1;
-
-		if( nlp->nlmsg_type == NLMSG_DONE )
-			break;
-
-		// increment the buffer pointer to place
-		// next message
-		p += rtn;
-
-		// increment the total size by the size of
-		// the last received message
-		nll += rtn;
-
-		printf( "XX : RTMSG %d RECEIVED ( %d BYTES )\n", nlp->nlmsg_type, rtn );
-	}
-
-	nlp = ( struct nlmsghdr * ) buf;
-
-	for( ; NLMSG_OK( nlp, nll ); nlp = NLMSG_NEXT( nlp, nll ) )
-	{
-		rtp = ( struct rtmsg * ) NLMSG_DATA( nlp );
-
-		if( rtp->rtm_table != RT_TABLE_MAIN )
-			continue;
-
-		rtap = ( struct rtattr * ) RTM_RTA( rtp );
-		rtl = RTM_PAYLOAD( nlp );
-
-		for( ; RTA_OK( rtap, rtl) ; rtap = RTA_NEXT( rtap, rtl ) )
+		if( nlmsg->nlmsg_type == RTM_NEWROUTE )
 		{
-			switch( rtap->rta_type )
+			// printf( "XX : netlink msg type = NLMSG_NEWROUTE\n" );
+
+			struct rtmsg * rtmsg = ( struct rtmsg * ) NLMSG_DATA( nlmsg );
+			int rtlen = RTM_PAYLOAD( nlmsg );
+
+			struct rtattr * rta = ( struct rtattr * ) RTM_RTA( rtmsg );
+
+			while( RTA_OK( rta, rtlen ) )
 			{
-				case RTA_DST:
+				switch( rta->rta_type )
 				{
-					memcpy( dst, RTA_DATA( rtap ), sizeof( *dst ) );
-					msk->s_addr = prefix_to_mask( rtp->rtm_dst_len );
-					break;
+					case RTA_DST:
+						// printf( "XX : netlink attribute = RTA_DST\n" );
+						memcpy( &route.addr, RTA_DATA( rta ), sizeof( route.addr ) );
+						route.mask.s_addr = prefix_to_mask( rtmsg->rtm_dst_len );
+						break;
+
+					case RTA_GATEWAY:
+						// printf( "XX : netlink attribute = RTA_GATEWAY\n" );
+						memcpy( &route.next, RTA_DATA( rta ), sizeof( route.next ) );
+						break;
+
+					case RTA_OIF:
+					{
+						// printf( "XX : netlink attribute = RTA_OIF\n" );
+
+						struct ifreq ifr;
+						int r = socket( PF_PACKET, SOCK_RAW, 0 );
+						if( r > 0 )
+						{
+							ifr.ifr_ifindex = *( ( int * ) RTA_DATA( rta ) );
+							ioctl( r, SIOCGIFNAME, &ifr );
+
+							ifr.ifr_addr.sa_family = AF_INET;
+							ioctl( r, SIOCGIFADDR, &ifr );
+
+							route.iface = ((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr;
+							close( r );
+						}
+						break;
+					}
+
+					case RTA_PREFSRC:
+						// printf( "XX : netlink attribute = RTA_PREFSRC\n" );
+						break;
+
+					case RTA_METRICS:
+						// printf( "XX : netlink attribute = RTA_METRICS\n" );
+						break;
+
+					case RTA_TABLE:
+						// printf( "XX : netlink attribute = RTA_TABLE\n" );
+						break;
+
+					case RTA_CACHEINFO:
+						// printf( "XX : netlink attribute = RTA_CACHEINFO\n" );
+						break;
+
+					default:
+						// printf( "XX : unhandled route attribute %i\n", rta->rta_type );
+						break;
 				}
 
-				case RTA_GATEWAY:
-					memcpy( gwy, RTA_DATA( rtap ), sizeof( *gwy ) );
-					break;
-
-				case RTA_OIF:
-				{
-					struct ifreq ifr;
-
-					int sock = socket( PF_PACKET, SOCK_RAW, 0 );
-
-					ifr.ifr_ifindex = *( ( int * ) RTA_DATA( rtap ) );
-					ioctl( sock, SIOCGIFNAME, &ifr );
-
-					ifr.ifr_addr.sa_family = AF_INET;
-					ioctl( sock, SIOCGIFADDR, &ifr );
-
-					memcpy( ifa, &((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr, sizeof( *ifa ) );
-
-					close( sock );
-				}
-
-				default:
-					break;
+				rta = RTA_NEXT( rta, rtlen );
 			}
+
+			return true;
 		}
+
+		if( nlmsg->nlmsg_type == RTM_DELROUTE )
+		{
+			// printf( "XX : netlink msg type = NLMSG_DELROUTE\n" );
+		}
+
+		if( nlmsg->nlmsg_type == RTM_GETROUTE )
+		{
+			// printf( "XX : netlink msg type = NLMSG_GETROUTE\n" );
+		}
+
+		if( nlmsg->nlmsg_type == NLMSG_ERROR )
+		{
+			// printf( "XX : netlink msg type = NLMSG_ERROR\n" );
+			break;
+		}
+
+		if( nlmsg->nlmsg_type == NLMSG_DONE )
+		{
+			// printf( "XX : netlink msg type = NLMSG_DONE\n" );
+			break;
+		}
+
+		nlmsg = NLMSG_NEXT( nlmsg, nllen );
 	}
 
-	return 0;
+	return false;
 }
 
 // add a route
 
-bool _IPROUTE::add( in_addr & iface, bool local, in_addr addr, in_addr mask, in_addr next )
+bool _IPROUTE::add( IPROUTE_ENTRY & route )
 {
 	// set route message header
 
@@ -563,26 +758,30 @@ bool _IPROUTE::add( in_addr & iface, bool local, in_addr addr, in_addr mask, in_
 	// add route destination
 
 	struct rtattr * rta = ( struct rtattr * ) nlmsg.buff;
-
 	rta->rta_type = RTA_DST;
-	rta->rta_len = sizeof( struct rtattr ) + sizeof( addr );
-	memcpy( ( ( char * ) rta ) + sizeof( struct rtattr ), &addr, sizeof( addr ) );
+	rta->rta_len = sizeof( struct rtattr );
+
+	struct in_addr * dst = ( in_addr * )( ( ( char * ) rta ) +  sizeof( struct rtattr ) );
+	*dst = route.addr;
+	rta->rta_len += sizeof( route.addr );
 
 	nlmsg.hdr.nlmsg_len += rta->rta_len;
 
 	// add route gateway
 
 	rta = ( struct rtattr * )( nlmsg.buff + nlmsg.hdr.nlmsg_len );
-
 	rta->rta_type = RTA_GATEWAY;
-	rta->rta_len = sizeof( struct rtattr ) + sizeof( next );
-	memcpy( ( ( char * ) rta ) + sizeof( struct rtattr ), &next, sizeof( next ) );
+	rta->rta_len = sizeof( struct rtattr );
+
+	struct in_addr * gwy = ( in_addr * )( ( ( char * ) rta ) +  sizeof( struct rtattr ) );
+	*gwy = route.next;
+	rta->rta_len += sizeof( route.next );
 
 	nlmsg.hdr.nlmsg_len += rta->rta_len;
 
 	// set route network mask
 
-	nlmsg.msg.rtm_dst_len = mask_to_prefix( mask );
+	nlmsg.msg.rtm_dst_len = mask_to_prefix( route.mask );
 
 	// set final message length
 
@@ -600,7 +799,7 @@ bool _IPROUTE::add( in_addr & iface, bool local, in_addr addr, in_addr mask, in_
 
 // delete a route
 
-bool _IPROUTE::del( in_addr & iface, bool local, in_addr addr, in_addr mask, in_addr next )
+bool _IPROUTE::del( IPROUTE_ENTRY & route )
 {
 	// set route message header
 
@@ -619,26 +818,30 @@ bool _IPROUTE::del( in_addr & iface, bool local, in_addr addr, in_addr mask, in_
 	// add route destination
 
 	struct rtattr * rta = ( struct rtattr * ) nlmsg.buff;
-
 	rta->rta_type = RTA_DST;
-	rta->rta_len = sizeof( struct rtattr ) + sizeof( addr );
-	memcpy( ( ( char * ) rta ) + sizeof( struct rtattr ), &addr, sizeof( addr ) );
+	rta->rta_len = sizeof( struct rtattr );
+
+	struct in_addr * dst = ( in_addr * )( ( ( char * ) rta ) +  sizeof( struct rtattr ) );
+	*dst = route.addr;
+	rta->rta_len += sizeof( route.addr );
 
 	nlmsg.hdr.nlmsg_len += rta->rta_len;
 
 	// add route gateway
 
 	rta = ( struct rtattr * )( nlmsg.buff + nlmsg.hdr.nlmsg_len );
-
 	rta->rta_type = RTA_GATEWAY;
-	rta->rta_len = sizeof( struct rtattr ) + sizeof( next );
-	memcpy( ( ( char * ) rta ) + sizeof( struct rtattr ), &next, sizeof( next ) );
+	rta->rta_len = sizeof( struct rtattr );
+
+	struct in_addr * gwy = ( in_addr * )( ( ( char * ) rta ) +  sizeof( struct rtattr ) );
+	*gwy = route.next;
+	rta->rta_len += sizeof( route.next );
 
 	nlmsg.hdr.nlmsg_len += rta->rta_len;
 
 	// set route network mask
 
-	nlmsg.msg.rtm_dst_len = mask_to_prefix( mask );
+	nlmsg.msg.rtm_dst_len = mask_to_prefix( route.mask );
 
 	// set final message length
 
@@ -656,33 +859,77 @@ bool _IPROUTE::del( in_addr & iface, bool local, in_addr addr, in_addr mask, in_
 
 // get a route ( by addr and mask )
 
-bool _IPROUTE::get( in_addr & iface, bool & local, in_addr & addr, in_addr & mask, in_addr & next )
-{
-	return true;
-}
-
-// get best route ( by address )
-
-bool _IPROUTE::best( in_addr & iface, bool & local, in_addr & addr, in_addr & mask, in_addr & next )
+bool _IPROUTE::get( IPROUTE_ENTRY & route )
 {
 	// set route message header
 
 	NLMSG nlmsg;
 	memset( &nlmsg, 0, sizeof( nlmsg ) );
 
-	nlmsg.hdr.nlmsg_flags = NLM_F_REQUEST | NLM_F_DUMP;
+	nlmsg.hdr.nlmsg_flags = NLM_F_REQUEST;
 	nlmsg.hdr.nlmsg_type = RTM_GETROUTE;
 
 	nlmsg.msg.rtm_family = AF_INET;
 	nlmsg.msg.rtm_table = RT_TABLE_MAIN;
+	nlmsg.msg.rtm_protocol = RTPROT_STATIC;
+	nlmsg.msg.rtm_scope = RT_SCOPE_UNIVERSE;
+	nlmsg.msg.rtm_type = RTN_UNICAST;
 
 	// add route destination
 
 	struct rtattr * rta = ( struct rtattr * ) nlmsg.buff;
-
 	rta->rta_type = RTA_DST;
-	rta->rta_len = sizeof( struct rtattr ) + sizeof( addr );
-	memcpy( ( ( char * ) rta ) + sizeof( struct rtattr ), &addr, sizeof( addr ) );
+	rta->rta_len = sizeof( struct rtattr );
+
+	struct in_addr * dst = ( in_addr * )( ( ( char * ) rta ) +  sizeof( struct rtattr ) );
+	*dst = route.addr;
+	rta->rta_len += sizeof( route.addr );
+
+	nlmsg.hdr.nlmsg_len += rta->rta_len;
+
+	// set route network mask
+
+	nlmsg.msg.rtm_dst_len = mask_to_prefix( route.mask );
+
+	// set final message length
+
+	nlmsg.hdr.nlmsg_len += sizeof( struct rtmsg );
+	nlmsg.hdr.nlmsg_len = NLMSG_LENGTH( nlmsg.hdr.nlmsg_len );
+
+	int s = rtmsg_send( &nlmsg );
+	if( s < 0 )
+		return false;
+
+	return rtmsg_recv( s, route );
+}
+
+// get best route ( by address )
+
+bool _IPROUTE::best( IPROUTE_ENTRY & route )
+{
+	// set route message header
+
+	NLMSG nlmsg;
+	memset( &nlmsg, 0, sizeof( nlmsg ) );
+
+	nlmsg.hdr.nlmsg_flags = NLM_F_REQUEST;
+	nlmsg.hdr.nlmsg_type = RTM_GETROUTE;
+
+	nlmsg.msg.rtm_family = AF_INET;
+	nlmsg.msg.rtm_table = RT_TABLE_UNSPEC;
+	nlmsg.msg.rtm_protocol = RTPROT_UNSPEC;
+	nlmsg.msg.rtm_scope = RT_SCOPE_UNIVERSE;
+	nlmsg.msg.rtm_type = RTN_UNSPEC;
+
+	// add route destination
+
+	struct rtattr * rta = ( struct rtattr * ) nlmsg.buff;
+	rta->rta_type = RTA_DST;
+	rta->rta_len = sizeof( struct rtattr );
+
+	struct in_addr * dst = ( in_addr * )( ( ( char * ) rta ) +  sizeof( struct rtattr ) );
+	*dst = route.addr;
+	rta->rta_len += sizeof( route.addr );
 
 	nlmsg.hdr.nlmsg_len += rta->rta_len;
 
@@ -699,32 +946,38 @@ bool _IPROUTE::best( in_addr & iface, bool & local, in_addr & addr, in_addr & ma
 	if( s < 0 )
 		return false;
 
-	int r = rtmsg_recv( s, &addr, &next, &mask, &iface );
-
-	close( s );
-
-	return ( r >= 0 );
+	return rtmsg_recv( s, route );
 }
 
-// decrement route metrics
+//
+// Linux systems appear to give priority to newer
+// routes so this is a no-op for now.
+//
+
+// decrement route costs
 
 bool _IPROUTE::increment( in_addr addr, in_addr mask )
 {
 	return true;
 }
 
-// increment route metrics
+// decrement route costs
 
 bool _IPROUTE::decrement( in_addr addr, in_addr mask )
 {
 	return true;
 }
 
+#endif
+
+//==============================================================================
+// Shared unix route functions
+//==============================================================================
+//
+
 // flush arp table
 
 bool _IPROUTE::flusharp( in_addr & iface )
 {
 	return true;
 }
-
-#endif
